@@ -100,6 +100,7 @@ static int cmdhlp(void)
 		printf("     r{0..64000}  : sample at specific rate\n");
 		printf("     8            : play/sample/mix as 8bit\n");
 		printf("     m            : play/sample/mix mono\n");
+		printf("-p                : quit when playlist is empty\n");
 		printf("\nExample : ocp -fl0,r1 -vp75,f2 -spdevpdisk -sr48000 ftstar.xm\n");
 		printf("          (for nice HD rendering of modules)\n");
 		return errHelpPrinted;
@@ -117,6 +118,7 @@ static int init_modules(int argc, char *argv[])
 	char const *regs;
 
 	void *reg;
+	int (*regmain)(int, char *[]);
 	int ret;
 
 	if ((ret=cmdhlp()))
@@ -128,7 +130,7 @@ static int init_modules(int argc, char *argv[])
 
 	cfConfigSec="defaultconfig";
 
-	if (cfGetProfileInt("version", "epoch", 0, 10)!=20050529)
+	if (cfGetProfileInt("version", "epoch", 0, 10)!=20060815)
 	{
 
 		if (isatty(2))
@@ -243,7 +245,8 @@ static int init_modules(int argc, char *argv[])
 	if (!reg)
 		return errSymSym;
 
-	if (((int (*)(int, char *[]))reg)(argc, argv)<0)
+	*((void **)&regmain)=reg;
+	if (regmain(argc, argv)<0)
 		return errGen;
 
 	plSetTextMode(255);
@@ -284,12 +287,3 @@ int bootup(int argc, char *argv[])
 
 	return 0;
 }
-
-
-int savebitrate(int br)
-{
-	static int b=0;
-	if(br>=0)
-		b=br;
-	return b;
-}
