@@ -19,11 +19,11 @@
 
 
 static void (*ChanDisplay)(uint16_t *buf, int len, int i);
-static short plChanFirstLine;
-static short plChanHeight;
-static short plChanWidth;
+static int plChanFirstLine;
+static int plChanHeight;
+static int plChanWidth;
 static char plChannelType;
-static short plChanStartCol;
+static int plChanStartCol;
 
 
 static void drawchannels()
@@ -96,7 +96,7 @@ static void drawchannels()
 				ChanDisplay(buf+8, 44, y+first);
 			}
 		}
-		displaystrattr(plChanFirstLine+y, /*(plChannelType==3)?80:0*/plChanStartCol, buf, plChanWidth);
+		displaystrattr(plChanFirstLine+y, plChanStartCol, buf, plChanWidth);
 	}
 }
 
