@@ -58,15 +58,8 @@
 #include <unistd.h>
 #include <time.h>
 
-#ifdef DEBUG
- int debugint;
- char debugstr[128];
-#endif
-
 #define MAXLCHAN 64
 
-unsigned char plScreenshotPal[768];
-
 void (*plGetRealMasterVolume)(int *l, int *r);
 
 extern struct cpimoderegstruct cpiModeText;
@@ -80,9 +73,9 @@ void (*plIdle)(void);
 
 char plMuteCh[MAXLCHAN];
 
-void (*plGetMasterSample)(int16_t *, int len, int rate, int mode);
-int (*plGetLChanSample)(int ch, int16_t *, int len, int rate, int opt);
-int (*plGetPChanSample)(int ch, int16_t *, int len, int rate, int opt);
+void (*plGetMasterSample)(int16_t *, int len, uint32_t rate, int mode);
+int (*plGetLChanSample)(int ch, int16_t *, int len, uint32_t rate, int opt);
+int (*plGetPChanSample)(int ch, int16_t *, int len, uint32_t rate, int opt);
 
 unsigned short plNLChan;
 unsigned short plNPChan;
@@ -98,7 +91,7 @@ char plPanType;
 static struct cpimoderegstruct *cpiModes;
 static struct cpimoderegstruct *cpiDefModes;
 
-unsigned short plTitleBuf[5][CONSOLE_MAX_X];
+uint16_t plTitleBuf[5][CONSOLE_MAX_X];
 static uint16_t plTitleBufOld[4][CONSOLE_MAX_X];
 
 unsigned long plEscTick;
@@ -122,7 +115,7 @@ void cpiSetTextMode(int size)
 void cpiDrawGStrings()
 {
 	char *verstr="  opencp v" VERSION;
-	char *author="(c) 1994-2005 Niklas Beisert et al.  ";
+	char *author="(c) 1994-2006 Stian Skjelstad ";
 	char tstr[CONSOLE_MAX_X+1];
 
 #ifdef DEBUG
@@ -164,10 +157,10 @@ void cpiDrawGStrings()
 		char lstr[CONSOLE_MAX_X+1];
 
 
-		strcpy(lstr, "      x  ");
+		strcpy(lstr, " \xc4 \xc4\xc4 \xc4\xc4\xc4 \xc4\xc4\xc4\xc4\xc4\xc4\xc4  x  ");
 		while (strlen(lstr)+10<plScrWidth)
-			strcat(lstr, "");
-		strcat(lstr, "    ");
+			strcat(lstr, "\xc4");
+		strcat(lstr, " \xc4\xc4\xc4 \xc4\xc4 \xc4 ");
 
 		writestring(plTitleBuf[4], 0, 0x08, lstr, plScrWidth);
 
@@ -177,7 +170,10 @@ void cpiDrawGStrings()
 				writenum(plTitleBuf[4], 16, 0x08, plScrWidth, 10, 3, 0);
 			else
 				writenum(plTitleBuf[4], 17, 0x08, plScrWidth, 10, 2, 0);
-		writenum(plTitleBuf[4], 20, 0x08, plScrHeight, 10, 2, 0);
+		if (plScrHeight>=100)
+			writenum(plTitleBuf[4], 20, 0x08, plScrHeight, 10, 3, 0);
+		else
+			writenum(plTitleBuf[4], 20, 0x08, plScrHeight, 10, 2, 0);
 
 		if (limit<2)
 			limit=2;
@@ -198,7 +194,7 @@ void cpiDrawGStrings()
 			unsigned short x;
 			x='0'+(i+chan0+1)%10;
 			if (plMuteCh[i+chan0]&&((i+chan0)!=plSelCh))
-				x=''|0x0800;
+				x='\xc4'|0x0800;
 			else
 				if (plMuteCh[i+chan0])
 					x|=0x8000;
@@ -634,6 +630,7 @@ static int plmpDrawScreen(void)
 				fsSetup();
 				plSetTextMode(fsScrType);
 				fsScrType=plScrType;
+				curmode->SetMode();
 				break;
 			/* TODO-keys case 0x2600: // ALT+l
 				plLoopPatterns=!plLoopPatterns;
@@ -645,7 +642,7 @@ static int plmpDrawScreen(void)
 			#endif
 				break;*/
 			#ifdef DOS32 /* TODO*/
-			case 0xF8: /* '' : screen shot */
+			case 0xF8:
 				Screenshot();
 				break;
 			#endif
