@@ -26,16 +26,16 @@
 
 static struct insdisplaystruct plInsDisplay;
 
-static short plInstScroll;
-static short plInstFirstLine;
-static short plInstStartCol;
-static short plInstLength;
-static short plInstHeight;
-static short plInstWidth;
+static int plInstScroll;
+static int plInstFirstLine;
+static int plInstStartCol;
+static int plInstLength;
+static int plInstHeight;
+static int plInstWidth;
 static char plInstType;
 static char plInstMode;
 
-static void displayshortins(char sel)
+static void displayshortins(int sel)
 {
 	int y,x,i;
 	uint16_t buf[40];
@@ -48,7 +48,7 @@ static void displayshortins(char sel)
 	{
 		if (y>=plInstLength)
 		{
-			displayvoid(y+plInstFirstLine, plInstStartCol/* 0 */, plInstWidth);
+			displayvoid(y+plInstFirstLine, plInstStartCol, plInstWidth);
 			continue;
 		}
 
@@ -67,7 +67,7 @@ static void displayshortins(char sel)
 	}
 }
 
-static void displayxshortins(char sel)
+static void displayxshortins(int sel)
 {
 	int y,x,i;
 	uint16_t buf[33];
@@ -98,7 +98,7 @@ static void displayxshortins(char sel)
 	}
 }
 
-static void displaysideins(char sel)
+static void displaysideins(int sel)
 {
 	int y;
 	uint16_t buf[52];
@@ -119,7 +119,7 @@ static void displaysideins(char sel)
 	}
 }
 
-static void displaylongins(char sel)
+static void displaylongins(int sel)
 {
 	int y;
 	uint16_t buf[80];
@@ -142,7 +142,7 @@ static void displaylongins(char sel)
 	}
 }
 
-static void displayxlongins(char sel)
+static void displayxlongins(int sel)
 {
 	int y;
 	uint16_t buf[132];
@@ -164,10 +164,7 @@ static void displayxlongins(char sel)
 	}
 }
 
-
-
-
-static void plDisplayInstruments(char sel)
+static void plDisplayInstruments(int sel)
 {
 	if (!plInstType)
 		return;
@@ -213,7 +210,7 @@ static void InstSetWin(int xpos, int wid, int ypos, int hgt)
 		else
 			plInstLength=(plInsDisplay.height+1)/(/* 2 */plInstWidth/40);
 	else if (plInstType==2)
-		plInstLength=plInsDisplay.height;
+		plInstLength=plInsDisplay.bigheight;
 	else
 		plInstLength=plInsDisplay.height;
 }
@@ -238,7 +235,7 @@ static int InstGetWin(struct cpitextmodequerystruct *q)
 			break;
 		case 2:
 			q->hgtmin=3;
-			q->hgtmax=2+plInsDisplay.height;
+			q->hgtmax=2+plInsDisplay.bigheight;
 			q->xmode=1;
 			break;
 		case 3:
@@ -261,7 +258,7 @@ static void InstDraw(int focus)
 	plDisplayInstruments(focus);
 }
 
-static int InstIProcessKey(unsigned short key)
+static int InstIProcessKey(uint16_t key)
 {
 	switch (key)
 	{
@@ -280,7 +277,7 @@ static int InstIProcessKey(unsigned short key)
 	return 1;
 }
 
-static int InstAProcessKey(unsigned short key)
+static int InstAProcessKey(uint16_t key)
 {
 	switch (key)
 	{
