@@ -24,19 +24,10 @@
  */
 
 #include "config.h"
-#ifdef __WATCOMC__
-#include <sys\types.h>
-#include <sys\stat.h>
-#include <io.h>
-#include <direct.h>
-#endif
-#ifndef __WATCOMC__
 #include <sys/types.h>
 #include <sys/stat.h>
 #include <unistd.h>
 #include <dirent.h>
-#endif
-
 #include <ctype.h>
 #include <fcntl.h>
 #include <stdlib.h>
@@ -46,7 +37,9 @@
 #include "boot/psetting.h"
 #include "cpiface.h"
 
-/*#include "gif.h"*/
+#ifdef HAVE_LZW
+#include "gif.h"
+#endif
 #include "tga.h"
 
 #include "boot/pmain.h"
@@ -115,7 +108,11 @@ void plReadOpenCPPic(void)
 			if(!match(name))
 				continue;
 			/* check for wildcard */
+#ifdef HAVE_LZW
 			if(!strncasecmp(name, "*.gif", 5) || !strncasecmp(name, "*.tga", 5))
+#else
+			if(!strncasecmp(name, "*.tga", 5))
+#endif
 			{
 				DIR *dir;
 				if(wildcardflag)
@@ -226,7 +223,9 @@ void plReadOpenCPPic(void)
 	}
 
 	/* read the picture */
+#ifdef HAVE_LZW
 	GIF87read(filecache, filesize, plOpenCPPict, plOpenCPPal, picWidth, picHeight);
+#endif
 	TGAread(filecache, filesize, plOpenCPPict, plOpenCPPal, picWidth, picHeight);
 
 	free(filecache);
