@@ -55,33 +55,33 @@ static int (*getpatlen)(int n);
 static const char *(*getpatname)(int n);
 static void (*seektrack)(int n, int c);
 static int (*startrow)();
-static int (*getnote)(unsigned short *bp, int small);
-static int (*getins)(unsigned short *bp);
-static int (*getvol)(unsigned short *bp);
-static int (*getpan)(unsigned short *bp);
-static void (*getfx)(unsigned short *bp, int n);
-static void (*getgcmd)(unsigned short *bp, int n);
+static int (*getnote)(uint16_t *bp, int small);
+static int (*getins)(uint16_t *bp);
+static int (*getvol)(uint16_t *bp);
+static int (*getpan)(uint16_t *bp);
+static void (*getfx)(uint16_t *bp, int n);
+static void (*getgcmd)(uint16_t *bp, int n);
 
 static int plPatternNum;
-static short plPrepdPat;
-static short plPatType;
-static short plPatFirstLine;
-static short plPatHeight;
-static short plPatWidth;
-static short plPatManualPat;
-static short plPatManualRow;
-static unsigned short (*plPatBuf)[CONSOLE_MAX_X];
-static unsigned short pathighlight[CONSOLE_MAX_X];
+static int plPrepdPat;
+static signed int  plPatType;
+static int plPatFirstLine;
+static int plPatHeight;
+static int plPatWidth;
+static int plPatManualPat;
+static int plPatManualRow;
+static uint16_t (*plPatBuf)[CONSOLE_MAX_X];
+static uint16_t pathighlight[CONSOLE_MAX_X];
 static char pattitle1[CONSOLE_MAX_X+1];
-static unsigned short pattitle2[CONSOLE_MAX_X];
-static short patwidth, patpad;
+static uint16_t pattitle2[CONSOLE_MAX_X];
+static int patwidth, patpad;
 
 enum
 {
 	cpiTrkFXIns=1,cpiTrkFXNote=2,cpiTrkFXVol=4,cpiTrkFXNoPan=8
 };
 
-static void getfx2(unsigned short *bp, int n, int o)
+static void getfx2(uint16_t *bp, int n, int o)
 {
 	int p=0;
 	if (o&cpiTrkFXIns)
@@ -141,9 +141,9 @@ static void getscrollpos(int tr, int *firstchan, int *chnn)
 }
 
 
-static void setattrgrey(unsigned short *buf, unsigned short len)
+static void setattrgrey(uint16_t *buf, int len)
 {
-	short i;
+	int i;
 	for (i=0; i<len; i++)
 	{
 		((char*)buf)[1]=COLMUTE;
@@ -151,35 +151,35 @@ static void setattrgrey(unsigned short *buf, unsigned short len)
 	}
 }
 
-static void preparetrack1(unsigned short *bp)
+static void preparetrack1(uint16_t *bp)
 {
 	getnote(bp, 2);
 }
 
-static void preparetrack2(unsigned short *bp)
+static void preparetrack2(uint16_t *bp)
 {
 	getnote(bp, 1);
 }
 
-static void preparetrack3(unsigned short *bp)
+static void preparetrack3(uint16_t *bp)
 {
 	getnote(bp, 0);
 }
 
-static void preparetrack3f(unsigned short *bp)
+static void preparetrack3f(uint16_t *bp)
 {
 	if (!getnote(bp, 0))
 		getfx2(bp, 1, cpiTrkFXVol);
 }
 
 
-static void preparetrack6nf(unsigned short *bp)
+static void preparetrack6nf(uint16_t *bp)
 {
 	getnote(bp, 0);
 	getfx2(bp+3, 1, cpiTrkFXVol);
 }
 
-static void preparetrack17invff(unsigned short *bp)
+static void preparetrack17invff(uint16_t *bp)
 {
 	getins(bp);
 	getnote(bp+3, 0);
@@ -187,7 +187,7 @@ static void preparetrack17invff(unsigned short *bp)
 	getfx2(bp+10, 2, 0);
 }
 
-static void preparetrack14invff(unsigned short *bp)
+static void preparetrack14invff(uint16_t *bp)
 {
 	getins(bp);
 	getnote(bp+2, 0);
@@ -195,14 +195,14 @@ static void preparetrack14invff(unsigned short *bp)
 	getfx2(bp+7, 2, 0);
 }
 
-static void preparetrack14nvff(unsigned short *bp)
+static void preparetrack14nvff(uint16_t *bp)
 {
 	getnote(bp, 0);
 	getvol(bp+4);
 	getfx2(bp+7, 2, 0);
 }
 
-static void preparetrack26invpffff(unsigned short *bp)
+static void preparetrack26invpffff(uint16_t *bp)
 {
 	getins(bp);
 	getnote(bp+3, 0);
@@ -211,19 +211,19 @@ static void preparetrack26invpffff(unsigned short *bp)
 	getfx2(bp+13, 4, cpiTrkFXNoPan);
 }
 
-static void preparetrack16fffff(unsigned short *bp)
+static void preparetrack16fffff(uint16_t *bp)
 {
 	getfx2(bp, 5, cpiTrkFXIns|cpiTrkFXNote|cpiTrkFXVol);
 }
 
-static void preparetrack8nvf(unsigned short *bp)
+static void preparetrack8nvf(uint16_t *bp)
 {
 	getnote(bp, 0);
 	getvol(bp+3);
 	getfx2(bp+5, 1, 0);
 }
 
-static void preparetrack8inf(unsigned short *bp)
+static void preparetrack8inf(uint16_t *bp)
 {
 	getins(bp);
 	getnote(bp+2, 0);
@@ -239,7 +239,7 @@ struct patviewtype
 	const char *paused;
 	const char *normal;
 	const char *selected;
-	void (*putcmd)(unsigned short *bp);
+	void (*putcmd)(uint16_t *bp);
 };
 
 static void preparepatgen(int pat, const struct patviewtype *pt)
@@ -248,8 +248,8 @@ static void preparepatgen(int pat, const struct patviewtype *pt)
 	int plen;
 	int firstchan,chnn;
 	const char *pname;
-	short p0, n0;
-	unsigned short patmask[CONSOLE_MAX_X];
+	int p0, n0;
+	uint16_t patmask[CONSOLE_MAX_X];
 	int firstrow=20;
 	int firstprow=0;
 	int firstpat;
@@ -283,7 +283,7 @@ static void preparepatgen(int pat, const struct patviewtype *pt)
 	patwidth=p0+pt->width*chnn+4;
 	writestring(pattitle2, 0, COLTITLE2, "row", CONSOLE_MAX_X);
 	if (patpad)
-		writestring(pattitle2, patwidth-3, COLTITLE2, "row", 3);
+		writestring(pattitle2, patwidth-4, COLTITLE2, "row", 3);
 	switch (pt->gcmd)
 	{
 		case 0:
@@ -306,11 +306,11 @@ static void preparepatgen(int pat, const struct patviewtype *pt)
 	writestring(patmask, 0, COLLNUM, "00 ", CONSOLE_MAX_X);
 	if (patpad)
 		writestring(patmask, patwidth-3, COLLNUM, "00", 2);
-	writestring(patmask, 3, COLBACK, "", 1);
-	writestring(patmask, 3+pt->gcmd*4, COLBACK, "", 1);
+	writestring(patmask, 3, COLBACK, "\xba", 1);
+	writestring(patmask, 3+pt->gcmd*4, COLBACK, "\xba", 1);
 	if (!plPause)
 		for (i=0; i<pt->gcmd; i++)
-			writestring(patmask, 4+4*i, COLBACK, "", 3);
+			writestring(patmask, 4+4*i, COLBACK, "\xfa\xfa\xfa", 3);
 
 	n0=p0+(pt->width+1)/2-1;
 	if (pt->width==4)
@@ -357,8 +357,8 @@ static void preparepatgen(int pat, const struct patviewtype *pt)
 		{
 			writestringattr(plPatBuf[i+firstrow-firstprow], 0, patmask, CONSOLE_MAX_X);
 			writenum(plPatBuf[i+firstrow-firstprow], 0, i?COLLNUM:COLHLNUM, i, 16, 2, 0);
-			if (patwidth<=128)
-			writenum(plPatBuf[i+firstrow-firstprow], patwidth-3, i?COLLNUM:COLHLNUM, i, 16, 2, 0);
+			if (patpad)
+				writenum(plPatBuf[i+firstrow-firstprow], patwidth-3, i?COLLNUM:COLHLNUM, i, 16, 2, 0);
 		}
 
 		if (pt->gcmd)
@@ -371,7 +371,7 @@ static void preparepatgen(int pat, const struct patviewtype *pt)
 					break;
 				if ((currow>=firstprow)&&(currow<lastprow))
 				{
-					unsigned short *bp=plPatBuf[currow+firstrow-firstprow]+4;
+					uint16_t *bp=plPatBuf[currow+firstrow-firstprow]+4;
 					getgcmd(bp, pt->gcmd);
 					if (plPause)
 						setattrgrey(bp, pt->gcmd*4);
@@ -391,7 +391,7 @@ static void preparepatgen(int pat, const struct patviewtype *pt)
 					break;
 				if ((currow>=firstprow)&&(currow<lastprow))
 				{
-					unsigned short *bp=plPatBuf[currow+firstrow-firstprow]+p0+i*pt->width;
+					uint16_t *bp=plPatBuf[currow+firstrow-firstprow]+p0+i*pt->width;
 					pt->putcmd(bp);
 					if (chpaus)
 						setattrgrey(bp, pt->width);
@@ -411,71 +411,71 @@ static void preparepatgen(int pat, const struct patviewtype *pt)
 }
 
 static struct patviewtype pat6480   = {/*64,*/  3,  1, "(notes only)",                          " ",
-                                                                                                "",
-                                                                                                "", preparetrack1};
+                                                                                                "\xfa",
+                                                                                                "\xf9", preparetrack1};
 static struct patviewtype pat64132  = {/*64,*/ 15,  1, "(notes only)",                          " ",
-                                                                                                "",
-                                                                                                "", preparetrack1};
+                                                                                                "\xfa",
+                                                                                                "\xf9", preparetrack1};
 static struct patviewtype pat64132m = {/*64,*/  0,  2, "(notes only)",                          "  ",
-                                                                                                "",
-                                                                                                "", preparetrack2};
+                                                                                                "\xfa\xfa",
+                                                                                                "\xf9\xf9", preparetrack2};
 static struct patviewtype pat4880   = {/*48,*/  6,  1, "(notes only)",                          " ",
-                                                                                                "",
-                                                                                                "", preparetrack1};
+                                                                                                "\xfa",
+                                                                                                "\xf9", preparetrack1};
 static struct patviewtype pat48132  = {/*48,*/  7,  2, "(notes only)",                          "  ",
-                                                                                                "",
-                                                                                                "", preparetrack2};
+                                                                                                "\xfa\xfa",
+                                                                                                "\xf9\xf9", preparetrack2};
 static struct patviewtype pat3280   = {/*32,*/  3,  2, "(notes only)",                          "  ",
-                                                                                                "",
-                                                                                                "", preparetrack2};
+                                                                                                "\xfa\xfa",
+                                                                                                "\xf9\xf9", preparetrack2};
 static struct patviewtype pat32132  = {/*32,*/  7,  3, "(notes only)",                          "   ",
-                                                                                                "",
-                                                                                                "", preparetrack3};
+                                                                                                "\xfa\xfa\xfa",
+                                                                                                "\xf9\xf9\xf9", preparetrack3};
 static struct patviewtype pat32132f = {/*32,*/  7,  3, "(notes & fx)",                          "   ",
-                                                                                                "",
-                                                                                                "", preparetrack3f};
+                                                                                                "\xfa\xfa\xfa",
+                                                                                                "\xf9\xf9\xf9", preparetrack3f};
 static struct patviewtype pat2480   = {/*24,*/  1,  3, "(notes only)",                          "   ",
-                                                                                                "",
-                                                                                                "", preparetrack3};
+                                                                                                "\xfa\xfa\xfa",
+                                                                                                "\xf9\xf9\xf9", preparetrack3};
 static struct patviewtype pat2480f  = {/*24,*/  1,  3, "(notes & fx)",                          "   ",
-                                                                                                "",
-                                                                                                "", preparetrack3f};
-static struct patviewtype pat24132  = {/*24,*/  7,  4, "(notes only)",                          "   ",
-                                                                                                "",
-                                                                                                "", preparetrack3};
-static struct patviewtype pat24132f = {/*24,*/  7,  4, "(notes & fx)",                          "   ",
-                                                                                                "",
-                                                                                                "", preparetrack3f};
-static struct patviewtype pat1680   = {/*16,*/  3,  4, "(notes only)",                          "   ",
-                                                                                                "",
-                                                                                                "", preparetrack3};
-static struct patviewtype pat1680f  = {/*16,*/  3,  4, "(notes & fx)",                          "   ",
-                                                                                                "",
-                                                                                                "", preparetrack3f};
-static struct patviewtype pat16132  = {/*16,*/  3,  7, "(note, fx)",                            "      ",
-                                                                                                "   ",
-                                                                                                "", preparetrack6nf};
-static struct patviewtype pat880    = {/* 8,*/  1,  9, "(ins, note, fx)",                       "        ",
-                                                                                                "     ",
-                                                                                                "", preparetrack8inf};
-static struct patviewtype pat880f   = {/* 8,*/  1,  9, "(note, vol, fx)",                       "        ",
-                                                                                                "  ",
-                                                                                                "", preparetrack8nvf};
-static struct patviewtype pat8132   = {/* 8,*/  3, 14, "(note, vol, fx, fx)",                   "             ",
-                                                                                                "  ",
-                                                                                                "  ", preparetrack14nvff};
-static struct patviewtype pat8132f  = {/* 8,*/  3, 14, "(ins, note, vol, fx, fx)",              "             ",
-                                                                                                "    ",
-                                                                                                "", preparetrack14invff};
-static struct patviewtype pat480    = {/* 4,*/  2, 17, "(ins, note, vol, fx, fx)",              "                ",
-                                                                                                "   ",
-                                                                                                "   ", preparetrack17invff};
-static struct patviewtype pat480f   = {/* 4,*/  3, 16, "(fx, fx, fx, fx, fx)",                  "               ",
-                                                                                                "",
-                                                                                                "", preparetrack16fffff};
-static struct patviewtype pat4132   = {/* 4,*/  5, 26, "(ins, note, vol, pan, fx, fx, fx, fx)", "                         ",
-	                                                                                        "    ",
-                                                                                                "    ", preparetrack26invpffff};
+                                                                                                "\xfa\xfa\xfa",
+                                                                                                "\xf9\xf9\xf9", preparetrack3f};
+static struct patviewtype pat24132  = {/*24,*/  7,  4, "(notes only)",                          "   \xb3",
+                                                                                                "\xfa\xfa\xfa\xb3",
+                                                                                                "\xf9\xf9\xf9\xb3", preparetrack3};
+static struct patviewtype pat24132f = {/*24,*/  7,  4, "(notes & fx)",                          "   \xb3",
+                                                                                                "\xfa\xfa\xfa\xb3",
+                                                                                                "\xf9\xf9\xf9\xb3", preparetrack3f};
+static struct patviewtype pat1680   = {/*16,*/  3,  4, "(notes only)",                          "   \xb3",
+                                                                                                "\xfa\xfa\xfa\xb3",
+                                                                                                "\xf9\xf9\xf9\xb3", preparetrack3};
+static struct patviewtype pat1680f  = {/*16,*/  3,  4, "(notes & fx)",                          "   \xb3",
+                                                                                                "\xfa\xfa\xfa\xb3",
+                                                                                                "\xf9\xf9\xf9\xb3", preparetrack3f};
+static struct patviewtype pat16132  = {/*16,*/  3,  7, "(note, fx)",                            "      \xb3",
+                                                                                                "\xfa\xfa\xfa   \xb3",
+                                                                                                "\xf9\xf9\xf9\xfa\xfa\xfa\xb3", preparetrack6nf};
+static struct patviewtype pat880    = {/* 8,*/  1,  9, "(ins, note, fx)",                       "        \xb3",
+                                                                                                "  \xfa\xfa\xfa   \xb3",
+                                                                                                "\xfa\xfa\xf9\xf9\xf9\xfa\xfa\xfa\xb3", preparetrack8inf};
+static struct patviewtype pat880f   = {/* 8,*/  1,  9, "(note, vol, fx)",                       "        \xb3",
+                                                                                                "\xfa\xfa\xfa  \xfa\xfa\xfa\xb3",
+                                                                                                "\xf9\xf9\xf9\xfa\xfa\xf9\xf9\xf9\xb3", preparetrack8nvf};
+static struct patviewtype pat8132   = {/* 8,*/  3, 14, "(note, vol, fx, fx)",                   "             \xb3",
+                                                                                                "\xfa\xfa\xfa \xfa\xfa \xfa\xfa\xfa\xfa\xfa\xfa\xb3",
+                                                                                                "\xf9\xf9\xf9 \xf9\xf9 \xf9\xf9\xf9\xf9\xf9\xf9\xb3", preparetrack14nvff};
+static struct patviewtype pat8132f  = {/* 8,*/  3, 14, "(ins, note, vol, fx, fx)",              "             \xb3",
+                                                                                                "  \xfa\xfa\xfa  \xfa\xfa\xfa\xfa\xfa\xfa\xb3",
+                                                                                                "\xfa\xfa\xf9\xf9\xf9\xfa\xfa\xf9\xf9\xf9\xf9\xf9\xf9\xb3", preparetrack14invff};
+static struct patviewtype pat480    = {/* 4,*/  2, 17, "(ins, note, vol, fx, fx)",              "                \xb3",
+                                                                                                "\xfa\xfa \xfa\xfa\xfa \xfa\xfa \xfa\xfa\xfa\xfa\xfa\xfa\xb3",
+                                                                                                "\xf9\xf9 \xf9\xf9\xf9 \xf9\xf9 \xf9\xf9\xf9\xf9\xf9\xf9\xb3", preparetrack17invff};
+static struct patviewtype pat480f   = {/* 4,*/  3, 16, "(fx, fx, fx, fx, fx)",                  "               \xb3",
+                                                                                                "\xfa\xfa\xfa\xfa\xfa\xfa\xfa\xfa\xfa\xfa\xfa\xfa\xfa\xfa\xfa\xb3",
+                                                                                                "\xf9\xf9\xf9\xf9\xf9\xf9\xf9\xf9\xf9\xf9\xf9\xf9\xf9\xf9\xf9\xb3", preparetrack16fffff};
+static struct patviewtype pat4132   = {/* 4,*/  5, 26, "(ins, note, vol, pan, fx, fx, fx, fx)", "                         \xb3",
+	                                                                                        "\xfa\xfa \xfa\xfa\xfa \xfa\xfa \xfa\xfa \xfa\xfa\xfa\xfa\xfa\xfa\xfa\xfa\xfa\xfa\xfa\xfa\xb3",
+                                                                                                "\xf9\xf9 \xf9\xf9\xf9 \xf9\xf9 \xf9\xf9 \xf9\xf9\xf9\xf9\xf9\xf9\xf9\xf9\xf9\xf9\xf9\xf9\xb3", preparetrack26invpffff};
 
 static int min(int a, int b)
 {
@@ -485,13 +485,13 @@ static int min(int a, int b)
 		return b;
 }
 
-static void gmdDrawPattern(char sel)
+static void gmdDrawPattern(int sel)
 {
 	int pos=getcurpos();
-	short pat=pos>>8;
+	int pat=pos>>8;
 	int currow=pos&0xFF;
 	int curpat=pat;
-	short crow=currow;
+	int crow=currow;
 	int i,j,row;
 	int plen;
 
@@ -573,7 +573,7 @@ static void gmdDrawPattern(char sel)
 	}
 }
 
-static int gmdTrkProcessKey(unsigned short key)
+static int gmdTrkProcessKey(uint16_t key)
 {
 	switch (key)
 	{
@@ -702,7 +702,7 @@ static void TrakDraw(int focus)
 	gmdDrawPattern(focus);
 }
 
-static int TrakIProcessKey(unsigned short key)
+static int TrakIProcessKey(uint16_t key)
 {
 	switch (key)
 	{
@@ -721,7 +721,7 @@ static int TrakIProcessKey(unsigned short key)
 	return 1;
 }
 
-static int TrakAProcessKey(unsigned short key)
+static int TrakAProcessKey(uint16_t key)
 {
 	switch (key)
 	{
@@ -743,7 +743,7 @@ static int trkEvent(int ev)
 			plTrackActive=cfGetProfileBool2(cfScreenSec, "screen", "pattern", 1, 1);
 			return 0;
 		case cpievInit:
-			plPatBuf=calloc(sizeof(short), (256+60)*CONSOLE_MAX_X);
+			plPatBuf=calloc(sizeof(short), (256+60+60/*don't ask where we get these from */)*CONSOLE_MAX_X);
 			if (!plPatBuf)
 				return 0;
 			break;
