@@ -8,9 +8,8 @@
  *    -first release
  */
 
-#define NO_MCPBASE_IMPORT
-
 #include "config.h"
+#include "types.h"
 #include "mcp.h"
 
 int mcpNChan;
@@ -24,9 +23,9 @@ void (*mcpSet)(int ch, int opt, int val);
 int (*mcpGet)(int ch, int opt);
 void (*mcpGetRealVolume)(int ch, int *l, int *r);
 void (*mcpGetRealMasterVolume)(int *l, int *r);
-void (*mcpGetMasterSample)(short *s, int len, int rate, int opt);
-int (*mcpGetChanSample)(int ch, short *s, int len, int rate, int opt);
-int (*mcpMixChanSamples)(int *ch, int n, short *s, int len, int rate, int opt);
+void (*mcpGetMasterSample)(int16_t *s, int len, uint32_t rate, int opt);
+int (*mcpGetChanSample)(int ch, int16_t *s, int len, uint32_t rate, int opt);
+int (*mcpMixChanSamples)(int *ch, int n, int16_t *s, int len, uint32_t rate, int opt);
 
 int mcpMixMaxRate;
 int mcpMixProcRate;
