@@ -24,7 +24,7 @@ int plrRate;
 int plrOpt;
 int (*plrPlay)(void **buf, int *len);
 void (*plrStop)(void);
-void (*plrSetOptions)(int rate, int opt);
+void (*plrSetOptions)(uint32_t rate, int opt);
 int (*plrGetBufPos)(void);
 int (*plrGetPlayPos)(void);
 void (*plrAdvanceTo)(int pos);
@@ -34,13 +34,13 @@ void (*plrIdle)(void);
 char *(*plrDebug)(void)=0;
 #endif
 
-static unsigned char stereo;
-static unsigned char bit16;
-static unsigned char reversestereo;
-static unsigned char signedout;
-static unsigned long samprate;
+static int stereo;
+static int bit16;
+static int reversestereo;
+static int signedout;
+static uint32_t samprate;
 
-static unsigned char *plrbuf;
+static uint8_t *plrbuf;
 static unsigned long buflen;
 
 
@@ -95,9 +95,9 @@ void plrGetRealMasterVolume(int *l, int *r)
 	}
 }
 
-void plrGetMasterSample(short *buf, int len, int rate, int opt)
+void plrGetMasterSample(int16_t *buf, int len, uint32_t rate, int opt)
 {
-	int step=imuldiv(samprate, 0x10000, rate);
+	uint32_t step=umuldiv(samprate, 0x10000, rate);
 	int maxlen;
 	int stereoout;
 	unsigned long bp;
@@ -113,7 +113,7 @@ void plrGetMasterSample(short *buf, int len, int rate, int opt)
 	stereoout=(opt&plrGetSampleStereo)?1:0;
 	if (len>maxlen)
 	{
-		memset((short*)buf+(maxlen<<stereoout), 0, (len-maxlen)<<(1+stereoout));
+		memset(buf+(maxlen<<stereoout), 0, (len-maxlen)<<(1+stereoout));
 		len=maxlen;
 	}
 
