@@ -18,11 +18,10 @@
  *     perfect (It is currently not hit, but I like it to be there still)
  */
 
-#define NO_MCPBASE_IMPORT
-
 #include "config.h"
 #include <stdlib.h>
 #include <stdio.h>
+#include "types.h"
 #include "mcp.h"
 
 #define SAMPEND 8
@@ -46,7 +45,7 @@ static void sampto8(struct sampleinfo *s)
 	s->type|=mcpSampRedBits;
 	l=(s->length+SAMPEND)<<sampsizefac(s->type);
 	for (i=0; i<l; i++)
-		((signed char *)s->ptr)[i]=((signed char *)s->ptr)[2*i+1];
+		((int8_t *)s->ptr)[i]=((int8_t *)s->ptr)[2*i+1];
        	s->ptr=realloc(s->ptr,(s->length+SAMPEND)<<sampsizefac(s->type));
 }
 
@@ -60,10 +59,10 @@ static void samptomono(struct sampleinfo *s)
 			((float *)s->ptr)[i]=(((float *)s->ptr)[2*i]+((float *)s->ptr)[2*i+1])/2;
 	else if (s->type&mcpSamp16Bit)
 		for (i=0; i<(s->length+SAMPEND); i++)
-			((signed short *)s->ptr)[i]=(((signed short *)s->ptr)[2*i]+((signed short *)s->ptr)[2*i+1])>>1;
+			((int16_t *)s->ptr)[i]=(((int16_t *)s->ptr)[2*i]+((int16_t *)s->ptr)[2*i+1])>>1;
 	else
 		for (i=0; i<(s->length+SAMPEND); i++)
-			((signed char *)s->ptr)[i]=(((signed char *)s->ptr)[2*i]+((signed char *)s->ptr)[2*i+1])>>1;
+			((int8_t *)s->ptr)[i]=(((int8_t *)s->ptr)[2*i]+((int8_t *)s->ptr)[2*i+1])>>1;
 	s->ptr=realloc(s->ptr,(s->length+SAMPEND)<<sampsizefac(s->type));
 }
 
@@ -74,19 +73,25 @@ static void samptofloat(struct sampleinfo *s)
 	int l=s->length<<sampsizefac(s->type&mcpSampStereo);
 	int l2;
 	float *newptr;
+	int extra;
+	
 	s->type|=mcpSampFloat;
 	l2=s->length<<sampsizefac(s->type);
 	s->ptr=realloc(s->ptr,l2);
-	newptr = malloc(sizeof(float)*(l+SAMPEND));
+	if (s->type&mcpSampStereo)
+		extra=SAMPEND*2;
+	else
+		extra=SAMPEND;
+	newptr = malloc(sizeof(float)*(l+extra));
 	if (s->type&mcpSamp16Bit)
 	{
 		for (i=0;i<l;i++)
-			newptr[i]=((signed short *)s->ptr)[i];
+			newptr[i]=((int16_t *)s->ptr)[i];
 	} else {
 		for (i=0;i<l;i++)
-			newptr[i]=257.0*((signed char *)s->ptr)[i];
+			newptr[i]=257.0*((int8_t *)s->ptr)[i];
 	}
-	for (i=0;i<SAMPEND;i++) /* TODO stereo */
+	for (i=0;i<extra;i++)
 		newptr[l+i]=newptr[l-1];
 	free(s->ptr);
 	s->ptr=newptr;
@@ -126,9 +131,9 @@ static int expandsmp(struct sampleinfo *s, int nopingpongloops)
 	int newlen=s->length;
 	int replen=s->loopend-s->loopstart;
 	int sreplen=s->sloopend-s->sloopstart;
-	char restricted=0;
-	char toforward=0;
-	char stoforward=0;
+	int restricted=0; /* boolean */
+	int toforward=0; /* boolean */
+	int stoforward=0; /* boolean */
 	int expandloop=0;
 	int sexpandloop=0;
 	int c;
@@ -179,7 +184,7 @@ static int expandsmp(struct sampleinfo *s, int nopingpongloops)
 	{
 		if (sampsizefac(s->type)==2)
 		{
-			long *p=(long *)s->ptr+s->loopend;
+			float *p=(float *)s->ptr+s->loopend;
 			for (c=s->length-s->loopend-1; c>=0; c--)
 				p[c+expandloop]=p[c];
 			if (!(s->type&mcpSampBiDi))
@@ -192,7 +197,7 @@ static int expandsmp(struct sampleinfo *s, int nopingpongloops)
 				p[c]=p[c-(replen<<1)];
 		} else if (sampsizefac(s->type)==1)
 		{
-			short *p=(short *)s->ptr+s->loopend;
+			int16_t *p=(int16_t *)s->ptr+s->loopend;
 			for (c=s->length-s->loopend-1; c>=0; c--)
 				p[c+expandloop]=p[c];
 			if (!(s->type&mcpSampBiDi))
@@ -204,7 +209,7 @@ static int expandsmp(struct sampleinfo *s, int nopingpongloops)
 			for (c=replen; c<expandloop; c++)
 				p[c]=p[c-(replen<<1)];
 		} else {
-			char *p=(char *)s->ptr+s->loopend;
+			int8_t *p=(int8_t *)s->ptr+s->loopend;
 			for (c=s->length-s->loopend-1; c>=0; c--)
 				p[c+expandloop]=p[c];
 			if (!(s->type&mcpSampBiDi))
@@ -233,7 +238,7 @@ static int expandsmp(struct sampleinfo *s, int nopingpongloops)
 	{
 		if (sampsizefac(s->type)==2)
 		{
-			long *p=(long *)s->ptr+s->sloopend;
+			float *p=(float *)s->ptr+s->sloopend;
 			for (c=0; c<(s->length-s->sloopend); c++)
 				p[c+sexpandloop]=p[c];
 			if (!(s->type&mcpSampSBiDi))
@@ -247,7 +252,7 @@ static int expandsmp(struct sampleinfo *s, int nopingpongloops)
 		} else
 			if (sampsizefac(s->type)==1)
 			{
-				short *p=(short *)s->ptr+s->sloopend;
+				int16_t *p=(int16_t *)s->ptr+s->sloopend;
 				for (c=0; c<(s->length-s->sloopend); c++)
 					p[c+sexpandloop]=p[c];
 				if (!(s->type&mcpSampSBiDi))
@@ -259,7 +264,7 @@ static int expandsmp(struct sampleinfo *s, int nopingpongloops)
 				for (c=sreplen; c<sexpandloop; c++)
 					p[c]=p[c-(sreplen<<1)];
 			} else {
-				char *p=(char *)s->ptr+s->sloopend;
+				int8_t *p=(int8_t *)s->ptr+s->sloopend;
 				for (c=0; c<(s->length-s->sloopend); c++)
 					p[c+sexpandloop]=p[c];
 				if (!(s->type&mcpSampSBiDi))
@@ -289,18 +294,18 @@ static int expandsmp(struct sampleinfo *s, int nopingpongloops)
 		if (!s->length)
 		{
 			if (sampsizefac(s->type)==2)
-				*(long*)s->ptr=0;
+				*(float *)s->ptr=0.0;
 			else if (sampsizefac(s->type)==1)
-				*(short*)s->ptr=0;
+				*(int16_t *)s->ptr=0;
 			else
-				*(char*)s->ptr=0;
+				*(int8_t *)s->ptr=0;
 		}
 		if (sampsizefac(s->type)==2)
-			((long*)s->ptr)[1]=*(long*)s->ptr;
+			((float *)s->ptr)[1]=*(float *)s->ptr;
 		else if (sampsizefac(s->type)==1)
-			((short*)s->ptr)[1]=*(short*)s->ptr;
+			((int16_t *)s->ptr)[1]=*(int16_t *)s->ptr;
 		else
-			((char*)s->ptr)[1]=*(char*)s->ptr;
+			((int8_t *)s->ptr)[1]=*(int8_t *)s->ptr;
 		s->length=2;
 	}
 	
@@ -319,7 +324,7 @@ static int repairsmp(struct sampleinfo *s)
 
 	if (sampsizefac(s->type)==2)
 	{
-		long *p=(long*)s->ptr;
+		float *p=(float *)s->ptr;
 		
 		for (i=0; i<SAMPEND; i++)
 			p[s->length+i]=p[s->length-1];
@@ -335,7 +340,7 @@ static int repairsmp(struct sampleinfo *s)
 		}
 	} else if (sampsizefac(s->type)==1)
 	{
-		short *p=(short*)s->ptr;
+		int16_t *p=(int16_t *)s->ptr;
 		for (i=0; i<SAMPEND; i++)
 			p[s->length+i]=p[s->length-1];
 		if ((s->type&mcpSampSLoop)&&!(s->type&mcpSampSBiDi))
@@ -349,7 +354,7 @@ static int repairsmp(struct sampleinfo *s)
 			p[s->loopend+1]=p[s->loopstart+1];
 		}
 	} else {
-		char *p=(char*)s->ptr;
+		int8_t *p=(int8_t *)s->ptr;
 		for (i=0; i<SAMPEND; i++)
 			p[s->length+i]=p[s->length-1];
 		if ((s->type&mcpSampSLoop)&&!(s->type&mcpSampSBiDi))
@@ -373,12 +378,12 @@ static void dividefrq(struct sampleinfo *s)
 	int i;
 	if (sampsizefac(s->type)==2)
 		for (i=0; i<(s->length>>1); i++)
-			((long*)s->ptr)[i]=((long*)s->ptr)[2*i];
+			((float *)s->ptr)[i]=((float *)s->ptr)[2*i];
 	else if (sampsizefac(s->type)==1)
 		for (i=0; i<(s->length>>1); i++)
-			((short*)s->ptr)[i]=((short*)s->ptr)[2*i];
+			((int16_t *)s->ptr)[i]=((int16_t *)s->ptr)[2*i];
 	else for (i=0; i<(s->length>>1); i++)
-		((char*)s->ptr)[i]=((char*)s->ptr)[2*i];
+		((int8_t *)s->ptr)[i]=((int8_t *)s->ptr)[2*i];
 
 	s->length>>=1;
 	s->loopstart>>=1;
@@ -403,11 +408,11 @@ static int totalsmpsize(struct sampleinfo *samples, int nsamp, int always16bit)
 	return curdif;
 }
 
-static int reduce16(struct sampleinfo *samples, int samplenum, unsigned long *redpars, int memmax)
+static int reduce16(struct sampleinfo *samples, int samplenum, uint32_t *redpars, int memmax)
 {
 	int i;
-	signed long curdif=-memmax;
-	signed long totdif=0;
+	int32_t curdif=-memmax;
+	int32_t totdif=0;
 	for (i=0; i<samplenum; i++)
 	{
 		struct sampleinfo *s=&samples[i];
@@ -431,7 +436,7 @@ static int reduce16(struct sampleinfo *samples, int samplenum, unsigned long *re
 	{
 		int fit=0;
 		int best=0; /* init to 0 removes warning */
-		long bestdif=0;
+		uint32_t bestdif=0;
 		for (i=0; i<samplenum; i++)
 			if (curdif<=redpars[i])
 			{
@@ -455,11 +460,11 @@ static int reduce16(struct sampleinfo *samples, int samplenum, unsigned long *re
 	return 1;
 }
 
-static int reducestereo(struct sampleinfo *samples, int samplenum, unsigned long *redpars, int memmax)
+static int reducestereo(struct sampleinfo *samples, int samplenum, uint32_t *redpars, int memmax)
 {
 	int i;
-	signed long curdif=-memmax;
-	signed long totdif=0;
+	int32_t curdif=-memmax;
+	int32_t totdif=0;
 	for (i=0; i<samplenum; i++)
 	{
 		struct sampleinfo *s=&samples[i];
@@ -483,7 +488,7 @@ static int reducestereo(struct sampleinfo *samples, int samplenum, unsigned long
 	{
 		int fit=0;
 		int best=0; /* init to 0 removes warning */
-		long bestdif=0;
+		uint32_t bestdif=0;
 		for (i=0; i<samplenum; i++)
 			if (curdif<=redpars[i])
 			{
@@ -507,10 +512,10 @@ static int reducestereo(struct sampleinfo *samples, int samplenum, unsigned long
 	return 1;
 }
 
-static int reducefrq(struct sampleinfo *samples, int samplenum, unsigned long *redpars, int memmax)
+static int reducefrq(struct sampleinfo *samples, int samplenum, uint32_t *redpars, int memmax)
 {
 	int i;
-	signed long curdif;
+	int32_t curdif;
 
 	for (i=-0x100; i<0x100; i++)
 		abstab[i+0x100]=i*i/16;
@@ -532,7 +537,7 @@ static int reducefrq(struct sampleinfo *samples, int samplenum, unsigned long *r
 	while (curdif>0)
 	{
 		int best=-1;
-		unsigned long bestpitch=0xFFFFFFFF;
+		uint32_t bestpitch=0xFFFFFFFF;
 		struct sampleinfo *s;
 		for (i=0; i<samplenum; i++)
 			if (redpars[i]<bestpitch)
@@ -567,17 +572,24 @@ static int convertsample(struct sampleinfo *s)
 	if (s->loopstart>=s->loopend)
 		s->type&=~mcpSampLoop;
 
-
+#if __BYTE_ORDER == __LITTLE_ENDIAN
 	if ((s->type&(mcpSampBigEndian|mcpSamp16Bit))==(mcpSampBigEndian|mcpSamp16Bit))
+#else
+	if ((s->type&(mcpSampBigEndian|mcpSamp16Bit))==(mcpSamp16Bit))
+#endif
 	{
 		int l=s->length<<sampsizefac(s->type);
 		for (i=0; i<l; i+=2)
 		{
-			char tmp=((char*)s->ptr)[i];
+			char tmp=((char *)s->ptr)[i];
 			((char*)s->ptr)[i]=((char*)s->ptr)[i+1];
 			((char*)s->ptr)[i+1]=tmp;
 		}
+#if __BYTE_ORDER == __LITTLE_ENDIAN
 		s->type&=~mcpSampBigEndian;
+#else
+		s->type|=mcpSampBigEndian;
+#endif
 	}
 	
 	if (s->type&mcpSampDelta)
@@ -586,32 +598,32 @@ static int convertsample(struct sampleinfo *s)
 		{
 			if (s->type&mcpSamp16Bit)
 			{
-				short oldl=0;
-				short oldr=0;
+				int16_t oldl=0;
+				int16_t oldr=0;
 				for (i=0; i<s->length; i++)
 				{
-					oldl=(((short*)s->ptr)[2*i]+=oldl);
-					oldr=(((short*)s->ptr)[2*i+1]+=oldr);
+					oldl=(((int16_t *)s->ptr)[2*i]+=oldl);
+					oldr=(((int16_t *)s->ptr)[2*i+1]+=oldr);
 				}
 			} else {
-				char oldl=0;
-				char oldr=0;
+				int8_t oldl=0;
+				int8_t oldr=0;
 				for (i=0; i<s->length; i++)
 				{
-					oldl=(((char*)s->ptr)[2*i]+=oldl);
-					oldr=(((char*)s->ptr)[2*i+1]+=oldr);
+					oldl=(((int8_t *)s->ptr)[2*i]+=oldl);
+					oldr=(((int8_t *)s->ptr)[2*i+1]+=oldr);
 				}
 			}
 		} else {
 			if (s->type&mcpSamp16Bit)
 			{
-				short old=0;
+				int16_t old=0;
 				for (i=0; i<s->length; i++)
-					old=(((short*)s->ptr)[i]+=old);
+					old=(((int16_t *)s->ptr)[i]+=old);
 			} else {
-				char old=0;
+				int8_t old=0;
 				for (i=0; i<s->length; i++)
-					old=(((char*)s->ptr)[i]+=old);
+					old=(((int8_t *)s->ptr)[i]+=old);
 			}
 		}
 		s->type&=~(mcpSampDelta|mcpSampUnsigned);
@@ -623,10 +635,10 @@ static int convertsample(struct sampleinfo *s)
 		if (s->type&mcpSamp16Bit)
 		{
 			for (i=0; i<l; i++)
-				((short *)s->ptr)[i]^=0x8000;
+				((int16_t *)s->ptr)[i]^=0x8000;
 		} else {
 			for (i=0; i<l; i++)
-				((char *)s->ptr)[i]^=0x80;
+				((int8_t *)s->ptr)[i]^=0x80;
 		}
 		s->type&=~mcpSampUnsigned;
 	}
@@ -637,7 +649,7 @@ static int convertsample(struct sampleinfo *s)
 int mcpReduceSamples(struct sampleinfo *si, int n, long mem, int opt)
 {
 	struct sampleinfo *samples=si;
-	unsigned long memmax=mem;
+	int32_t memmax=mem;
 	int samplenum=n;
 
 	int i;
@@ -662,7 +674,7 @@ int mcpReduceSamples(struct sampleinfo *si, int n, long mem, int opt)
 	
 	if (totalsmpsize(samples, samplenum, opt&mcpRedAlways16Bit)>memmax)
 	{
-		unsigned long *redpars=malloc(sizeof(unsigned long)*samplenum);
+		uint32_t *redpars=malloc(sizeof(uint32_t)*samplenum);
 		if (!redpars)
 			return 0;
 		if ((opt&mcpRedAlways16Bit)||!reduce16(samples, samplenum, redpars, memmax))
