@@ -16,6 +16,7 @@
  */
 
 #include "config.h"
+#include <errno.h>
 #include <fcntl.h>
 #include <stdlib.h>
 #include <string.h>
@@ -52,15 +53,23 @@ static void Flush(void)
 	{
 		if (!writeerr)
 		{
+rewrite:
+			if (bit16)
+			{
+				int i, j = cachepos/2;
+				uint16_t *d = (uint16_t *)diskcache;
+				for (i=0;i<j;i++)
+					d[i]=uint16_little(d[i]);
+			}
 			if (write(file, diskcache, cachepos)!=cachepos)
 			{
+				if (errno==EAGAIN)
+					goto rewrite;
+				if (errno==EINTR)
+					goto rewrite;
 				writeerr=1;
 			}
 		}
-		if (writeerr)
-		{
-			printf("\a");
-		}
 		filepos+=cachepos;
 		cachepos=0;
 	}
@@ -89,6 +98,12 @@ static void advance(int pos)
 		cachepos+=pos-bufpos;
 	}
 
+	if (cachepos>cachelen)
+	{
+		fprintf(stderr, "devpdisk: cachepos>cachelen\n");
+		exit(0);
+	}
+
 	bufpos=pos;
 	busy=0;
 }
@@ -98,7 +113,7 @@ static long gettimer(void)
 	return imuldiv((filepos+cachepos), (65536>>(stereo+bit16)), playrate);
 }
 
-static void dwSetOptions(int rate, int opt)
+static void dwSetOptions(uint32_t rate, int opt)
 {
 	/* don't even THINK about removing this line! */
 	if (opt&PLR_RESTRICTED)
@@ -155,19 +170,24 @@ static int dwPlay(void **buf, int *len)
 		fn[5]+=(i/100)%10;
 		fn[6]+=(i/10)%10;
 		fn[7]+=(i/1)%10;
-		file=open(fn, O_RDONLY);
-		if (file==-1)
+		if ((file=open(fn, O_WRONLY|O_CREAT|O_EXCL, S_IREAD|S_IWRITE))>=0)
 			break;
-		close(file);
 	}
 
-	if ((file=creat(fn, S_IREAD|S_IWRITE))<0)
-	{
-		perror("creat()");
+	if (file<0)
 		return 0;
+
+	while (1)
+	{
+		if (write(file, hdr, 0x2C)>=0)
+			break;
+		if (errno==EAGAIN)
+			continue;
+		if (errno==EINTR)
+			continue;
+		break;
 	}
-	
-	write(file, hdr, 0x2C);
+			
 	buflen=*len;
 	bufpos=0;
 	busy=0;
@@ -187,28 +207,38 @@ static int dwPlay(void **buf, int *len)
 
 static void dwStop(void)
 {
-	unsigned long wavlen;
-	struct
+	uint32_t wavlen;
+	struct __attribute__((packed))
 	{
 		char riff[4];
-		unsigned long len24;
+		uint32_t len24;
 		char wave[4];
 		char fmt_[4];
-		unsigned long chlen;
-		unsigned short form;
-		unsigned short chan;
-		unsigned long rate;
-		unsigned long datarate;
-		unsigned short bpsmp;
-		unsigned short bits;
+		uint32_t chlen;
+		uint16_t form;
+		uint16_t chan;
+		uint32_t rate;
+		uint32_t datarate;
+		uint16_t bpsmp;
+		uint16_t bits;
 		char data[4];
-		unsigned long wavlen;
+		uint32_t wavlen;
 	} wavhdr;
 
 	plrIdle=0;
 
 	if (!writeerr)
-		write(file, diskcache, cachepos);
+	{
+rewrite:
+		if (write(file, diskcache, cachepos)<0)
+		{
+			if (errno==EINTR)
+				goto rewrite;
+			if (errno==EAGAIN)
+				goto rewrite;
+		}
+	}
+		
 
 	wavlen=lseek(file, 0, SEEK_CUR)-0x2C;
 
@@ -218,19 +248,31 @@ static void dwStop(void)
 	memcpy(wavhdr.wave, "WAVE", 4);
 	memcpy(wavhdr.fmt_, "fmt ", 4);
 	memcpy(wavhdr.data, "data", 4);
-	wavhdr.chlen=0x10;
-	wavhdr.form=1;
-	wavhdr.chan=1<<stereo;
-	wavhdr.rate=playrate;
-	wavhdr.bits=8<<bit16;
-	wavhdr.bpsmp=wavhdr.chan*wavhdr.bits/8;
-	wavhdr.datarate=wavhdr.bpsmp*wavhdr.rate;
-	wavhdr.wavlen=wavlen;
-	wavhdr.len24=wavhdr.wavlen+0x24;
-	write(file, &wavhdr, 0x2C);
+	wavhdr.chlen=uint32_little(0x10);
+	wavhdr.form=uint16_little(1);
+	wavhdr.chan=uint16_little(1<<stereo);
+	wavhdr.rate=uint32_little(playrate);
+	wavhdr.bits=uint16_little(8<<bit16);
+	wavhdr.bpsmp=uint16_little((1<<stereo)*(8<<bit16)/8);
+	wavhdr.datarate=uint32_little(((1<<stereo)*(8<<bit16)/8)*playrate);
+	wavhdr.wavlen=uint32_little(wavlen);
+	wavhdr.len24=uint32_little(wavlen+0x24);
+rewrite2:
+	if (write(file, &wavhdr, 0x2C)<0)
+	{
+		if (errno==EINTR)
+			goto rewrite2;
+		if (errno==EAGAIN)
+			goto rewrite2;
+	}
 
 	lseek(file, 0, SEEK_END);
-	close(file);
+reclose:
+	if (close(file)<0)
+	{
+		if (errno==EINTR)
+			goto reclose;
+	}
 	free(playbuf);
 	free(diskcache);
 }
