@@ -14,15 +14,17 @@
 
 #include "config.h"
 #include <errno.h>
-#include <fcntl.h>
-#include <sys/soundcard.h>
 #include <stdio.h>
 #include <stdlib.h>
-#include <sys/ioctl.h>
-#include <sys/stat.h>
 #include <sys/types.h>
 #include <string.h>
+#include <fcntl.h>
+#include <sys/ioctl.h>
+#include <sys/stat.h>
 #include <unistd.h>
+#ifdef HAVE_SYS_SOUNDCARD_H
+#include <sys/soundcard.h>
+#endif
 #include "types.h"
 
 #include "boot/plinkman.h"
@@ -83,6 +85,7 @@ static void flush(void)
 		return;		
 	}
 
+#ifdef SNDCTL_DSP_GETODELAY
 	/* update kernlen stuff before we check if cachelen is set furter down */
 	if (ioctl(fd_dsp, SNDCTL_DSP_GETODELAY, &odelay))
 	{
@@ -92,7 +95,29 @@ static void flush(void)
 		busy--;
 		return;
 	}
-
+#ifdef OSS_DEBUG
+	{
+		if (odelay<0)
+		{
+			write(2, "devposs: buggy nvidia driver?\n", 30);
+		}
+	}
+#endif
+#else
+	{
+		struct count_info i;
+		if (ioctl(fd_dsp, SNDCTL_DSP_GETOPTR, &i))
+		{
+#ifdef OSS_DEBUG
+			perror("devposs: flush() ioctl(OSS, SNDCTL_DSP_GETOPTR, &i)");
+#endif
+			busy--;
+			return;
+		}
+		odelay=i.bytes;
+	}
+#endif
+	odelay=abs(odelay); /* This is because of nvidia sound-drivers */
 	if (odelay>kernlen)
 	{
 #ifdef OSS_DEBUG
@@ -176,7 +201,7 @@ static int getplaypos(void)
 #endif
 	} else {
 		int tmp;
-		
+#ifdef SNDCTL_DSP_GETODELAY
 		if (ioctl(fd_dsp, SNDCTL_DSP_GETODELAY, &tmp))
 		{
 #ifdef OSS_DEBUG
@@ -185,6 +210,30 @@ static int getplaypos(void)
 			busy--;
 			return kernpos;
 		}
+#ifdef OSS_DEBUG
+		{
+			if (tmp<0)
+			{
+				write(2, "devposs: buggy nvidia driver?\n", 30);
+			}
+		}
+#else
+		{
+			struct count_info i;
+			if (ioctl(fd_dsp, SNDCTL_DSP_GETOPTR, &i))
+			{
+#ifdef OSS_DEBUG
+				perror("devposs: flush() ioctl(OSS, SNDCTL_DSP_GETOPTR, &i)");
+#endif
+				busy--;
+				return kernpos;
+			}
+			tmp=i.bytes;
+		}
+#endif
+
+#endif
+		tmp=abs(tmp); /* we love nvidia, don't we */
 		if (tmp>kernlen)
 		{
 #ifdef OSS_DEBUG
@@ -247,6 +296,19 @@ static void advance(int pos)
 #endif
 	}
 
+	if (bit16)
+	{
+		int i;
+		i=bufpos;
+		while (i!=pos)
+		{
+			((uint16_t *)playbuf)[i>>1]=uint16_little(((uint16_t *)playbuf)[i>>1]);
+			i+=2;
+			if (i>=buflen)
+				i=0;
+		}
+	}
+
 	cachelen+=(pos-bufpos+buflen)%buflen;
 	bufpos=pos;
 
@@ -270,6 +332,7 @@ static long gettimer(void)
 #endif
 		odelay=kernlen;
 	} else {
+#ifdef SNDCTL_DSP_GETODELAY
 		if (ioctl(fd_dsp, SNDCTL_DSP_GETODELAY, &odelay))
 		{
 #ifdef OSS_DEBUG
@@ -277,6 +340,29 @@ static long gettimer(void)
 #endif
 			odelay=kernlen;
 		}
+#ifdef OSS_DEBUG
+		{
+			if (odelay<0)
+			{
+				write(2, "devposs: buggy nvidia driver?\n", 30);
+			}
+		}
+#endif
+#else
+		{
+			struct count_info i;
+			if (ioctl(fd_dsp, SNDCTL_DSP_GETOPTR, &i))
+		{
+#ifdef OSS_DEBUG
+				perror("devposs: flush() ioctl(OSS, SNDCTL_DSP_GETOPTR, &i)");
+#endif
+				odelay=kernlen;
+			} else
+				odelay=i.bytes;
+		}
+#endif
+
+		odelay=abs(odelay);
 		if (odelay>kernlen)
 		{
 #ifdef OSS_DEBUG
@@ -378,7 +464,7 @@ static int ossPlay(void **buf, int *len)
 	return 1;
 }
 
-static void SetOptions(int rate, int opt)
+static void SetOptions(uint32_t rate, int opt)
 {
 	int tmp;
 
@@ -449,7 +535,9 @@ static void SetOptions(int rate, int opt)
 
 static int ossDetect(struct deviceinfo *card)
 {
+#ifdef HAVE_SYS_SOUNDCARD_H
 	struct stat st;	
+#endif
 	int tmp;
 	char *temp;
 
@@ -482,6 +570,8 @@ static int ossDetect(struct deviceinfo *card)
 		strncpy(card->mixer, temp, 128);
 		card->mixer[127]=0;
 	}
+#ifdef HAVE_SYS_SOUNDCARD_H
+/* this test will fail with liboss */
 	if (stat(card->path, &st))
 	{
 #ifdef OSS_DEBUG
@@ -489,14 +579,15 @@ static int ossDetect(struct deviceinfo *card)
 #endif
 		return 0;
 	}
+#endif
 
 	/* test if basic OSS functions exists */
 	if ((fd_dsp=open(card->path, O_WRONLY|O_NONBLOCK))<0) /* no need to FD_SETFD, since it will be closed soon */
 	{
-		if (errno==EAGAIN)
+		if ((errno==EAGAIN)||(errno==EINTR))
 		{
 #ifdef OSS_DEBUG
-			fprintf(stderr, "devposs: OSS detected (EAGAIN)\n");
+			fprintf(stderr, "devposs: OSS detected (EAGAIN/EINTR)\n");
 #endif
 			return 1;
 		}
