@@ -37,10 +37,6 @@
 #include <math.h>
 #include <unistd.h>
 #include <sys/mman.h>
-#include <limits.h>    /* for PAGESIZE */
-#ifndef PAGESIZE
-#define PAGESIZE 4096
-#endif
 #include "types.h"
 #include "boot/plinkman.h"
 #include "dev/imsdev.h"
@@ -50,7 +46,6 @@
 #include "stuff/imsrtns.h"
 #include "devwmixf.h"
 #include "stuff/poll.h"
-
 #include "dwmixfa.h"
 
 /* NB NB   more includes further down in the file */
@@ -64,7 +59,7 @@ static uint32_t pausesamps;
 static struct sampleinfo *samples;
 static int samplenum;
 
-static volatile int clipbusy;
+static volatile int clipbusy=0;
 static float amplify;
 static float transform[2][2];
 static int   volopt;
@@ -105,13 +100,27 @@ static int   masterchr;
 
 inline float frchk(float a)
 {
+#if 1
+#warning This is not portable
+	union {
+		float a;
+		uint32_t b;
+	} cast;
+	cast.a=a;
+	cast.b&=0x7f800000;
+	cast.b>>=23;
+	if (cast.b==0 || cast.b==0x000000ff)
+		return 0.0;
+#else
+#warning This is not portable
 	uint32_t b=*((uint32_t *)(void *)&a); /* makes compilers shut up warnings */
 	b&=0x7f800000;
 	b>>=23;
 	if (b==0 || b==0xff)
-		return 0;
+		return 0.0;
+#endif
 	if (fabs(a)<0.00000001)
-		return 0;
+		return 0.0;
 	return a;
 }
 
@@ -279,11 +288,10 @@ static void setlbuf(struct channel *c)
 
 
 
-static void mixmain(int min)
+static void mixmain(/*int min*/)
 {
 	int i;
-	int bufmax,
-	    bufmin,
+	int /*bufmin,*/
 	    bufdeltatot;
 
   #ifdef RASTER
@@ -301,16 +309,14 @@ static void mixmain(int min)
 		return;
 	}
 
-	bufmax=imulshr16(mcpMixMax, samprate);
-	if (bufmax>buflen)
-		bufmax=buflen;
-	bufmin=bufmax-imulshr16(min, samprate);
-	if (bufmin<0)
-		bufmin=0;
 
-	bufdeltatot=((buflen+(plrGetBufPos()>>(stereo+bit16))-bufpos)%buflen)-buflen+bufmax;
-	if (bufdeltatot<bufmin)
-		bufdeltatot=0;
+/*	bufmin=imulshr16(min, samprate);
+	if (bufmin<0)
+		bufmin=0;*/
+
+	bufdeltatot=((buflen+(plrGetBufPos()>>(stereo+bit16))-bufpos)%buflen);
+/*	if (bufdeltatot<bufmin)
+		bufdeltatot=0;*/
 
 	if (dopause)
 	{
@@ -446,16 +452,14 @@ static void mixmain(int min)
   #endif
 }
 
-#ifndef WIN32
 static void timerproc()
 {
 #ifndef NO_BACKGROUND_MIXER
-	mixmain(mcpMixPoll);
+	mixmain(/*mcpMixPoll*/);
 #endif
 	if (plrIdle)
 		plrIdle();
 }
-#endif
 
 static void SET(int ch, int opt, int val)
 {
@@ -737,7 +741,7 @@ static int GET(int ch, int opt)
 
 static void Idle(void)
 {
-	mixmain(mcpMixMax);
+	mixmain(/*mcpMixMax*/);
 	if (plrIdle)
 		plrIdle();
 }
@@ -752,11 +756,12 @@ static void Idle(void)
  * bla, toller hack: es funktioniert. (fd)
  */
 
-static void GetMixChannel(int ch, struct mixchannel *chn, int rate)
+static void GetMixChannel(int ch, struct mixchannel *chn, uint32_t rate)
 {
 	struct channel *c=&channels[ch];
 
 	chn->samp=c->samp;
+	chn->realsamp.fmt=c->samp;
 	chn->length=c->length;
 	chn->loopstart=c->loopstart;
 	chn->loopend=c->loopend;
@@ -764,8 +769,8 @@ static void GetMixChannel(int ch, struct mixchannel *chn, int rate)
 	chn->pos=smpposw[ch]-(float*)c->samp;
 	chn->vol.volfs[0]=fabs(c->vol[0]);
 	chn->vol.volfs[1]=fabs(c->vol[1]);
-	chn->step=imuldiv((freqw[ch]<<16)|(freqf[ch]>>16), samprate, rate); 
-	chn->status=MIX_PLAY32BIT;
+	chn->step=imuldiv((freqw[ch]<<16)|(freqf[ch]>>16), samprate, (signed)rate); 
+	chn->status=MIX_PLAYFLOAT;
 	if (voiceflags[ch]&MIXF_MUTE)
 		chn->status|=MIX_MUTE;
 	if (voiceflags[ch]&MIXF_LOOPED)
@@ -873,7 +878,6 @@ static int OpenPlayer(int chan, void (*proc)(void))
 	tickplayed=0;
 	cmdtimerpos=0;
 
-  #ifndef WIN32
 	if (!pollInit(timerproc))
 	{
 		mcpNChan=0;
@@ -882,7 +886,6 @@ static int OpenPlayer(int chan, void (*proc)(void))
 		mixClose();
 		return 0;
 	}
-  #endif
 	{
 		struct mixfpostprocregstruct *mode;
 
@@ -899,9 +902,7 @@ static void ClosePlayer()
 	  mcpNChan=0;
 	  mcpIdle=0;
 
-#ifndef WIN32
 	  pollClose();
-#endif
 
 	  plrClosePlayer();
 
@@ -918,6 +919,7 @@ static void ClosePlayer()
 
 static int Init(const struct deviceinfo *dev)
 {
+#ifdef I386_ASM
 	/* Self-modifying code needs access to modify it self */
 	{
 		int fd;
@@ -961,6 +963,7 @@ static int Init(const struct deviceinfo *dev)
 		unlink(file);
 		free(file);
 	}
+#endif
 
 	volramp=!!(dev->opt&MIXF_VOLRAMP);
 	declick=!!(dev->opt&MIXF_DECLICK);
