@@ -10,7 +10,7 @@ struct adbregstruct
 {
 	const char *ext;
 	int (*Scan)(const char *path);
-	int (*Call)(int act, const char *apath, const char *fullname, const char *file, const char *dpath);
+	int (*Call)(const int act, const char *apath, const char *fullname, const int fd);
 	struct adbregstruct *next;
 };
 
@@ -29,9 +29,13 @@ struct __attribute__((packed)) arcentry
 extern char adbInit(void);
 extern void adbUpdate(void);
 extern void adbClose(void);
+extern int adbCallArc(const char *cmd, const char *arc, const char *name, const char *dir);
 
 extern int adbAdd(const struct arcentry *a);
 extern uint32_t adbFind(const char *arcname);
-extern char isarchivepath(const char *p);
+extern int isarchivepath(const char *p);
+
+extern const char adbsigv1[16];
+extern const char adbsigv2[16];
 
 #endif
