@@ -60,6 +60,13 @@ struct __attribute__((packed)) modinfoentry
 #define gen mie.gen
 #define comp mie.comp
 
+struct __attribute__((packed)) mdbheader
+{
+	char sig[60];
+	uint32_t entries;
+};
+const char mdbsigv1[60] = "Cubic Player Module Information Data Base\x1B\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00";
+
 static struct modinfoentry *mdbData;
 static uint32_t mdbNum;
 static int mdbDirty;
@@ -184,7 +191,7 @@ int mdbWriteModuleInfo(uint32_t fileref, struct moduleinfostruct *m)
 	if ((mdbData[fileref].flags&(MDB_USED|MDB_BLOCKTYPE))!=(MDB_USED|MDB_GENERAL))
 		return 0;
 
-	m->flags1=MDB_USED|MDB_DIRTY|MDB_GENERAL|(m->flags1&(MDB_VIRTUAL|MDB_BIGMODULE));
+	m->flags1=MDB_USED|MDB_DIRTY|MDB_GENERAL|(m->flags1&(MDB_VIRTUAL|MDB_BIGMODULE|MDB_PLAYLIST));
 	m->flags2=MDB_DIRTY|MDB_COMPOSER;
 	m->flags3=MDB_DIRTY|MDB_COMMENT;
 	m->flags4=MDB_DIRTY|MDB_FUTURE;
@@ -236,6 +243,8 @@ void mdbScan(struct modlistentry *m)
 	{
 		struct moduleinfostruct mdbEditBuf;
 		FILE *f;
+		if (m->flags&MODLIST_FLAG_VIRTUAL) /* don't scan virtual files */
+			return;
 		if (!(f=m->ReadHandle(m)))
 			return;
 		mdbGetModuleInfo(&mdbEditBuf, m->fileref);
@@ -245,12 +254,33 @@ void mdbScan(struct modlistentry *m)
 	}
 }
 
-static void mdbRegisterReadDir(struct mdbreaddirregstruct *r)
+void mdbRegisterReadDir(struct mdbreaddirregstruct *r)
 {
 	r->next=mdbReadDirs;
 	mdbReadDirs=r;
 }
 
+void mdbUnregisterReadDir(struct mdbreaddirregstruct *r)
+{
+	struct mdbreaddirregstruct *root=mdbReadDirs;
+	if (root==r)
+	{
+		mdbReadDirs=r->next;
+		return;
+	}
+	while (1)
+	{
+		if (root->next==r)
+		{
+			root->next=root->next->next;
+			return;
+		}
+		if (!root->next)
+			return;
+		root=root->next;
+	}
+}
+
 static int miecmp(const void *a, const void *b)
 {
 	struct modinfoentry *c=&mdbData[*(uint32_t *)a];
@@ -269,7 +299,7 @@ int mdbInit(void)
 	const char *regs;
 	char path[PATH_MAX+1];
 	int f;
-	char sig[64];
+	struct mdbheader header;
 	uint32_t i;
 
 	regs=_lnkReadInfoReg("readdirs");
@@ -282,7 +312,7 @@ int mdbInit(void)
 			fprintf(stderr, "mdb: Registering %s\n", regname);
 #endif
 			mdbRegisterReadDir((struct mdbreaddirregstruct *)reg);
-#ifdef LD_DBUG
+#ifdef LD_DEBUG
 		} else {
 			fprintf(stderr, "mdb: UNABLE TO LOCATE %s\n", regname);
 #endif
@@ -323,25 +353,28 @@ int mdbInit(void)
 	strcat(path, "CPMODNFO.DAT");
 
 	if ((f=open(path, O_RDONLY))<0)
+	{
+		perror("open(cfConfigDir/CPMODNDO.DAT)");
 		return 1;
+	}
 
 	fprintf(stderr, "Loading %s .. ", path);
 
-	if (read(f, sig, 64)!=64)
+	if (read(f, &header, sizeof(header))!=sizeof(header))
 	{
 		fprintf(stderr, "No header\n");
 		close(f);
 		return 1;
 	}
 
-	if (memcmp(sig, "Cubic Player Module Information Data Base\x1B\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00", 60))
+	if (memcmp(header.sig, mdbsigv1, sizeof(mdbsigv1)))
 	{
 		fprintf(stderr, "Invalid header\n");
      		close(f);
 		return 1;
 	}
 
-	mdbNum=*(unsigned long*)(sig+60);
+	mdbNum=uint32_little(header.entries);
 	if (!mdbNum)
 	{
 		close(f);
@@ -388,6 +421,7 @@ void mdbUpdate(void)
 	char path[PATH_MAX+1];
 	int f;
 	uint32_t i, j;
+	struct mdbheader header;
 
 	if (!mdbDirty||!fsWriteModInfo)
 		return;
@@ -408,10 +442,9 @@ void mdbUpdate(void)
 	}
 
 	lseek(f, 0, SEEK_SET);
-	write(f, "Cubic Player Module Information Data Base\x1B", 42);
-	write(f, "\x00\x00", 2);
-	write(f, "\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00", 16);
-	write(f, &mdbNum, 4);
+	memcpy(header.sig, mdbsigv1, sizeof(mdbsigv1));
+	header.entries = uint32_little(mdbNum);
+	write(f, &header, sizeof(header));
 	
 	i=0;
 	while (i<mdbNum)
@@ -535,18 +568,3 @@ invalid:
 		memcpy(&m->flags4, mdbData+m->futref, 70);
 	return 1;
 }
-
-#if 0
-superset by ml->append(ml, &f);
-int mdbAppend(modlist &m, const modlistentry &f)
-{
-  return m.append(f);
-}
-
-int mdbAppendNew(modlist &m, const modlistentry &f)
-{
-  if (m.find(f)!=-1)
-    return 1;
-  return m.append(f);
-}
-#endif
