@@ -10,6 +10,7 @@ struct __attribute__((packed)) moduleinfostruct
 #define MDB_BLOCKTYPE 12
 #define MDB_VIRTUAL 16
 #define MDB_BIGMODULE 32
+#define MDB_PLAYLIST 64 /* handle as dir */
 	uint8_t flags1;
 #define MDB_GENERAL 0
 	/* sorry dudes, but I upgraded the references to 32 bit */
@@ -46,6 +47,7 @@ enum
 	mtMID=16, mtCDA=17, mtMIDd=18, mtPTM=19, mtMED=20, mtMDL=21, mtAMS=22,
 	mtINP=23, mtDEVp=24, mtDEVs=25, mtDEVw=26, mtIT=27, mtWAV=28, mtVOC=29,
 	mtMPx=30, mtSID=31, mtMXM=32, mtMODf=33, mtWMA=34, mtOGG=35, mtOPL=36,
+	mtAY=37,
 	mtPLS=128, mtM3U=129, mtANI=130,
 	mtUnRead=0xFF
 };
@@ -87,6 +89,8 @@ extern void mdbClose(void);
 extern uint32_t mdbGetModuleReference(const char *name, uint32_t size);
 extern int mdbGetModuleInfo(struct moduleinfostruct *m, uint32_t fileref);
 
+extern void mdbRegisterReadDir(struct mdbreaddirregstruct *r);
+extern void mdbUnregisterReadDir(struct mdbreaddirregstruct *r);
 
 #if 0
 /* these two are super-set by modlist itself */
@@ -94,4 +98,6 @@ extern int mdbAppend(struct modlist *m, const struct modlistentry *f);
 extern int mdbAppendNew(struct modlist *m, const struct modlistentry *f);
 #endif
 
+extern const char mdbsigv1[60];
+
 #endif
