@@ -38,12 +38,12 @@ static void modlist_free(struct modlist *modlist)
 		free(modlist->files);
 	free(modlist);
 }
-static struct modlistentry *modlist_getcur(struct modlist *modlist) /* does not Ref() */
+static struct modlistentry *modlist_getcur(const struct modlist *modlist) /* does not Ref() */
 {
 	return modlist->get(modlist, modlist->pos);
 }
 
-static struct modlistentry *modlist_get(struct modlist *modlist, int index) /* does not Ref() */
+static struct modlistentry *modlist_get(const struct modlist *modlist, int index) /* does not Ref() */
 {
 	if (!(modlist->num))
 		return NULL; /* should never happen */
@@ -77,10 +77,13 @@ static void modlist_append(struct modlist *modlist, struct modlistentry *entry)
 
 static void modlist_remove(struct modlist *modlist, int index, int count)
 {
+	int i;
 	if (index>=modlist->num)
 		return;
 	if (index+count>modlist->num)
 		count=modlist->num-index;
+	for (i=index;i<(index+count);i++)
+		free(modlist->files[i]);
 	memmove(&modlist->files[index], &modlist->files[index+count], (modlist->num-index-count)*sizeof(modlist->files[0]));
 	modlist->num-=count;
 	if ((modlist->max-modlist->num>75))
