@@ -4,26 +4,26 @@
 #include <stdio.h> /* FILE * */
 
 struct moduleinfostruct;
-extern char fsGetNextFile(char *path, struct moduleinfostruct *info, FILE **); /* info comes from external buffer */
-extern char fsFilesLeft(void);
-extern signed char fsFileSelect(void);
+extern int fsGetNextFile(char *path, struct moduleinfostruct *info, FILE **); /* info comes from external buffer */
+extern int fsFilesLeft(void);
+extern signed int fsFileSelect(void);
 /* extern char fsAddFiles(const char *);      use the playlist instead..*/
 extern char fsInit(void);
 extern void fsClose(void);
 
-extern char fsListScramble;
-extern char fsListRemove;
-extern char fsLoopMods;
-extern char fsScanNames;
-extern char fsScanArcs;
-extern char fsScanInArc;
-extern char fsScanMIF;
-extern char fsScrType;
-extern char fsEditWin;
-extern char fsColorTypes;
-extern char fsInfoMode;
-extern char fsPutArcs;
-extern char fsWriteModInfo;
+extern int fsListScramble;
+extern int fsListRemove;
+extern int fsLoopMods;
+extern int fsScanNames;
+extern int fsScanArcs;
+extern int fsScanInArc;
+extern int fsScanMIF;
+extern int fsScrType;
+extern int fsEditWin;
+extern int fsColorTypes;
+extern int fsInfoMode;
+extern int fsPutArcs;
+extern int fsWriteModInfo;
 extern const char *(fsTypeNames[256]); /* type description */
 
 extern void fsRegisterExt(const char *ext);
@@ -50,8 +50,7 @@ struct interfacestruct
 #define RD_PUTSUBS 1
 #define RD_ARCSCAN 2
 #define RD_DIRRECURSE 4
-#define RD_PUTDSUBS 16
-#define RD_PUTRSUBS 32
+#define RD_PUTRSUBS 16
 
 #if 0
 struct modlist;
@@ -74,11 +73,14 @@ struct fsgetfileregstruct
 struct dmDrive
 {
 	char drivename[13];
-	char currentpath[PATH_MAX+1];
+	/*char currentpath[PATH_MAX+1];*/
+	uint32_t basepath;
+	uint32_t currentpath;
 	struct dmDrive *next;
 };
 extern struct dmDrive *dmDrives;
-extern void RegisterDrive(const char *dmDrive);
+extern struct dmDrive *RegisterDrive(const char *dmDrive);
+extern struct dmDrive *dmFindDrive(const char *dmDrive); /* to get the correct drive from a given string */
 
 void fsConvFileName12(char *c, const char *f, const char *e);
 void convfilename12wc(char *c, const char *f, const char *e);
