@@ -537,5 +537,5 @@ struct initcloseregstruct fsReg={fsint, fsclose};
 struct filehandlerstruct fsAddPLS={addpls};
 struct filehandlerstruct fsAddM3U={addm3u};
 */
-char *dllinfo = "initcloseafter fsReg; main fsMain; readdirs adbReadDirReg dosReadDirReg fsReadDirReg; getfile adbGetFileReg dosGetFileReg; readinfos fsReadInfoReg";
-struct linkinfostruct dllextinfo = {"cfilesel.so", "OpenCP Fileselector (c) 1994-04 Niklas Beisert, Tammo Hinrichs", DLLVERSION, 0};
+char *dllinfo = "initcloseafter fsReg; main fsMain; readdirs adbReadDirReg dosReadDirReg fsReadDirReg plsReadDirReg m3uReadDirReg; getfile adbGetFileReg dosGetFileReg; readinfos fsReadInfoReg";
+struct linkinfostruct dllextinfo = {"pfilesel", "OpenCP Fileselector (c) 1994-04 Niklas Beisert, Tammo Hinrichs", DLLVERSION, 0};
