@@ -80,6 +80,7 @@
 #include <stdlib.h>
 #include <stdio.h>
 #include <string.h>
+#include <zlib.h>
 #include "types.h"
 #include "boot/psetting.h"
 #include "stuff/poutput.h"
@@ -88,18 +89,17 @@
 #include "boot/pmain.h"
 #include "help/cphelper.h"
 #include "binfile/binfpak.h"
-#include "filesel/inflate.h"
 #include <curses.h>
 
 static int plWinFirstLine, plWinHeight, plHelpHeight, plHelpScroll;
 
-static int       Helpfile_ID=1213219663;
-static int       Helpfile_Ver=0x011000;
-static int       Helppages;
-static int       HelpfileErr=hlpErrNoFile;
-static helppage *Page, *curpage;
-static link     *curlink;
-static int       link_ind;
+static int        Helpfile_ID=1213219663;
+static int        Helpfile_Ver=0x011000;
+static uint32_t   Helppages;
+static int        HelpfileErr=hlpErrNoFile;
+static helppage  *Page, *curpage;
+static help_link *curlink;
+static int        link_ind;
 
 /* Helpfile Commands */
 
@@ -139,7 +139,9 @@ static int doReadVersion100Helpfile(FILE *file)
 		fread(Page[i].desc, len, 1, file);
 
 		fread(&Page[i].size, sizeof(Page[i].size), 1, file);
+		Page[i].size = uint32_little (Page[i].size);
 		fread(&Page[i].lines, sizeof(Page[i].lines), 1, file);
+		Page[i].lines = uint32_little (Page[i].lines);
 
 		Page[i].links=NULL;
 		Page[i].rendered=NULL;
@@ -161,6 +163,7 @@ static int doReadVersion110Helpfile(FILE *file)
 	int i;
 
 	fread(&Helppages, sizeof(Helppages), 1, file);
+	Helppages = uint32_little (Helppages);
 	Page = calloc(Helppages, sizeof(Page[0]));
 
 	compdatasize=calloc(Helppages, sizeof(int));
@@ -178,7 +181,9 @@ static int doReadVersion110Helpfile(FILE *file)
 		fread(Page[i].desc, len, 1, file);
 
 		fread(&Page[i].size, sizeof(Page[i].size), 1, file);
+		Page[i].size = uint32_little (Page[i].size);
 		fread(&Page[i].lines, sizeof(Page[i].lines), 1, file);
+		Page[i].lines = uint32_little (Page[i].lines);
 
 		fread(&compdatasize[i], sizeof(compdatasize[i]), 1, file);
 		Page[i].links=NULL;
@@ -187,11 +192,12 @@ static int doReadVersion110Helpfile(FILE *file)
 
 	for (i=0; i<Helppages; i++)
 	{
+		uLongf temp=Page[i].size;
 		Page[i].data=calloc(Page[i].size, 1);
 		inbuf=calloc(compdatasize[i], 1);
 		fread(inbuf, compdatasize[i], 1, file);
-		inflate(Page[i].data, inbuf);
-/*		_inflate(inbuf, compdatasize[i], Page[i].data, Page[i].size);*/
+		uncompress((Bytef *)Page[i].data, &temp, (Bytef *)inbuf, compdatasize[i]);
+		Page[i].size=temp;
 		free(inbuf);
 	}
 	free(compdatasize);
@@ -201,14 +207,16 @@ static int doReadVersion110Helpfile(FILE *file)
 
 static int doReadHelpFile(FILE *file)
 {
-	int version;
+	uint32_t version;
+	uint32_t temp;
 
-	int temp;
 	fread(&temp, sizeof(temp), 1, file);
+	temp = uint32_little (temp);
 	if (temp!=Helpfile_ID)
 		return hlpErrBadFile;
 
 	fread(&version, sizeof(version), 1, file);
+	version = uint32_little (version);
 
 	if (version>Helpfile_Ver)
 		return hlpErrTooNew;
@@ -235,7 +243,7 @@ static char plReadHelpExternal(void)
 		return 1;
 
 	strcpy(helpname, cfDataDir);
-	strcat(helpname, "cp.hlp");
+	strcat(helpname, "ocp.hlp");
 
 	if ((bf=fopen(helpname, "r")))
 	{
@@ -255,7 +263,7 @@ static char plReadHelpPack(void)
 	if (Page && (HelpfileErr==hlpErrOk))
 		return 1;
 
-	if ((ref=pakfOpen("cp.hlp")))
+	if ((ref=pakfOpen("ocp.hlp")))
 	{
 		HelpfileErr=doReadHelpFile(ref);
 		fclose(ref);
@@ -276,14 +284,14 @@ helppage *brDecodeRef(char *name)
 	return NULL;
 }
 
-static link *firstLinkOnPage(helppage *pg)
+static help_link *firstLinkOnPage(helppage *pg)
 {
 	if (!pg->linkcount)
 		return NULL;
 	return &pg->links[0];
 }
 
-static int linkOnCurrentPage(link *lnk)
+static int linkOnCurrentPage(help_link *lnk)
 {
 	int y;
 
@@ -452,7 +460,7 @@ void brRenderPage(helppage *pg)
 		};
 	};
 
-	pg->links=calloc(sizeof(link), lcount);
+	pg->links=calloc(sizeof(help_link), lcount);
 	pg->linkcount=lcount;
 
 	for (i=0; i<lcount; i++)
@@ -551,10 +559,10 @@ void brDisplayHelp(void)
 		switch (HelpfileErr)
 		{
 			case hlpErrNoFile:
-				strcat(errormsg, "Helpfile \"CP.HLP\" is not present");
+				strcat(errormsg, "Helpfile \"OCP.HLP\" is not present");
 				break;
 			case hlpErrBadFile:
-				strcat(errormsg, "Helpfile \"CP.HLP\" is corrupted");
+				strcat(errormsg, "Helpfile \"OCP.HLP\" is corrupted");
 				break;
 			case hlpErrTooNew:
 				strcat(errormsg, "Helpfile version is too new. Please update.");
@@ -617,19 +625,19 @@ static void processActiveHyperlink(void)
 		brSetPage((helppage *) curlink->ref);
 }
 
-void brSetWinStart(short fl)
+void brSetWinStart(int fl)
 {
 	plWinFirstLine=fl;
 }
 
-void brSetWinHeight(short h)
+void brSetWinHeight(int h)
 {
 	plWinHeight=h;
 }
 
-int brHelpKey(unsigned short key)
+int brHelpKey(uint16_t key)
 {
-	link *link2;
+	help_link *link2;
 
 	if(!curpage)
 		return 1;
