@@ -11,43 +11,105 @@
  *    -Linux related changes
  */
 
-#include <linux/cdrom.h>
 #include "config.h"
 #include <ctype.h>
 #include <stdio.h>
 #include <string.h>
+#include <linux/cdrom.h>
 #include "types.h"
 #include "cdaudio.h"
 #include "boot/plinkman.h"
-#include "boot/psetting.h"
 #include "cpiface/cpiface.h"
-#include "filesel/pfilesel.h"
+#include "dev/deviplay.h"
 #include "dev/devisamp.h"
-#include "dev/sampler.h"
+#include "filesel/pfilesel.h"
 #include "filesel/mdb.h"
+#include "stuff/compat.h"
 #include "stuff/poutput.h"
+#include "stuff/sets.h"
 
 #include <curses.h>
 
 void _splitpath(const char *src, char *drive, char *path, char *file, char *ext); /* defined in stuff/compat.h but this header clashed with linux/cdrom.h */
 
-static int nodevice; /* do we have a devs */
-static char cfCDAtLineIn;
 static unsigned long cdpTrackStarts[CDROM_LEADOUT+1];
 static unsigned char cdpFirstTrack;
 static unsigned char cdpPlayMode; /* 1 = disk, 0 = track */
 static unsigned char cdpTrackNum; /* current track, calculated in GStrings */
-static unsigned char cdpViewSectors; /* ??? view-option */
+static int cdpViewSectors; /* ??? view-option */
 static FILE *cdpDrive; /* nasty */
 static unsigned long basesec; /* current start... usually a cdpTrack[n] value */
 static unsigned long length; /* and the length of it */
 static signed long newpos; /* skip to */
 static unsigned char setnewpos; /* and the fact we should skip */
-static unsigned long curpos; /* current sector */
 static char vdev[8];
 
 uint16_t status;
 
+static int16_t speed;
+static char finespeed=8;
+static uint32_t pausefadestart;
+static uint8_t pausefaderelspeed;
+static int8_t pausefadedirect;
+
+static void startpausefade(void)
+{
+	if (pausefadedirect)
+	{
+		if (pausefadedirect<0)
+			plPause=1;
+		pausefadestart=2*dos_clock()-DOS_CLK_TCK-pausefadestart;
+	} else
+		pausefadestart=dos_clock();
+
+	if (plPause)
+	{
+		plChanChanged=1;
+		plPause=0;
+		cdRestart(fileno(cdpDrive));
+		pausefadedirect=1;
+	} else
+		pausefadedirect=-1;
+}
+
+static void normalize(void)
+{
+	mcpNormalize(0);
+	speed=set.speed;
+	cdSetSpeed(speed);
+}
+
+static void dopausefade(void)
+{
+	int16_t i;
+	if (pausefadedirect>0)
+	{
+		i=((int32_t)dos_clock()-pausefadestart)*64/DOS_CLK_TCK;
+		if (i<0)
+			i=0;
+		if (i>=64)
+		{
+			i=64;
+			pausefadedirect=0;
+		}
+	} else {
+		i=64-((int32_t)dos_clock()-pausefadestart)*64/DOS_CLK_TCK;
+		if (i>=64)
+			i=64;
+		if (i<=0)
+		{
+			i=0;
+			pausefadedirect=0;
+			plPause=1;
+			cdPause(fileno(cdpDrive));
+			plChanChanged=1;
+			cdSetSpeed(speed);
+			return;
+		}
+	}
+	pausefaderelspeed=i;
+	cdSetSpeed(speed*i/64);
+}
 static char *gettimestr(unsigned long s, char *time)
 {
 	unsigned char csec, sec, min;
@@ -68,33 +130,40 @@ static char *gettimestr(unsigned long s, char *time)
 
 static void cdaDrawGStrings(uint16_t (*buf)[CONSOLE_MAX_X])
 {
+	int i;
+	char timestr[9];
+
+	struct cdStat stat;
+
+	cdGetStatus(fileno(cdpDrive), &stat);
+
+
 	memset(buf[0]+80, 0, (plScrWidth-80)*sizeof(uint16_t));
 	memset(buf[1]+80, 0, (plScrWidth-80)*sizeof(uint16_t));
 	memset(buf[2]+80, 0, (plScrWidth-80)*sizeof(uint16_t));
 
 	writestring(buf[0], 0, 0x09, "  mode: ..........  ", 20);
       	writestring(buf[0], 8, 0x0F, "cd-audio", 10);
-	int i;
-
-	char timestr[9];
 	for (i=1; i<=cdpTrackNum; i++)
-		if (curpos<cdpTrackStarts[i])
+		if (stat.position<cdpTrackStarts[i])
 			break;
 
 	writestring(buf[0], 20, 0x09, "playmode: .....  status: .......", plScrWidth-20);
 	writestring(buf[0], 30, 0x0F, cdpPlayMode?"disk ":"track", 5);
-	writestring(buf[0], 45, 0x0F, (status&STATUS_ERROR)?"  error":(status&STATUS_PLAY)?"playing":" paused", 7);
+	writestring(buf[0], 45, 0x0F, (stat.error)?"  error":(stat.paused)?" paused":"playing", 7);
+	writestring(buf[0], 52, 0x0F, "                 speed: ...%", 28);
+	_writenum(buf[0], 76, 0x0F, stat.speed*100/256, 10, 3);
 
 	writestring(buf[1], 0, 0x09, "drive: ....... start:   :..:..  pos:   :..:..  length:   :..:..  size: ...... kb", plScrWidth);
 	writestring(buf[1], 7, 0x0F, vdev, 7); /* VERY TODO.. can we fit more data on this line??? */
 	if (cdpViewSectors)
 	{
 		writenum(buf[1], 22, 0x0F, cdpTrackStarts[0], 10, 8, 0);
-		writenum(buf[1], 37, 0x0F, curpos-cdpTrackStarts[0], 10, 8, 0);
+		writenum(buf[1], 37, 0x0F, stat.position-cdpTrackStarts[0], 10, 8, 0);
 		writenum(buf[1], 55, 0x0F, cdpTrackStarts[cdpTrackNum]-cdpTrackStarts[0], 10, 8, 0);
 	} else {
 		writestring(buf[1], 22, 0x0F, gettimestr(cdpTrackStarts[0]+150, timestr), 8);
-		writestring(buf[1], 37, 0x0F, gettimestr(curpos-cdpTrackStarts[0], timestr), 8);
+		writestring(buf[1], 37, 0x0F, gettimestr(stat.position-cdpTrackStarts[0], timestr), 8);
 		writestring(buf[1], 55, 0x0F, gettimestr(cdpTrackStarts[cdpTrackNum]-cdpTrackStarts[0], timestr), 8);
 	}
 	_writenum(buf[1], 71, 0x0F, (cdpTrackStarts[cdpTrackNum]-cdpTrackStarts[0])*147/64, 10, 6);
@@ -104,11 +173,11 @@ static void cdaDrawGStrings(uint16_t (*buf)[CONSOLE_MAX_X])
 	if (cdpViewSectors)
 	{
 		writenum(buf[2], 22, 0x0F, cdpTrackStarts[i-1]+150, 10, 8, 0);
-		writenum(buf[2], 37, 0x0F, curpos-cdpTrackStarts[i-1], 10, 8, 0);
+		writenum(buf[2], 37, 0x0F, stat.position-cdpTrackStarts[i-1], 10, 8, 0);
 		writenum(buf[2], 55, 0x0F, cdpTrackStarts[i]-cdpTrackStarts[i-1], 10, 8, 0);
 	} else {
 		writestring(buf[2], 22, 0x0F, gettimestr(cdpTrackStarts[i-1]+150, timestr), 8);
-		writestring(buf[2], 37, 0x0F, gettimestr(curpos-cdpTrackStarts[i-1], timestr), 8);
+		writestring(buf[2], 37, 0x0F, gettimestr(stat.position-cdpTrackStarts[i-1], timestr), 8);
 		writestring(buf[2], 55, 0x0F, gettimestr(cdpTrackStarts[i]-cdpTrackStarts[i-1], timestr), 8);
 	}
 	_writenum(buf[2], 71, 0x0F, (cdpTrackStarts[i]-cdpTrackStarts[i-1])*147/64, 10, 6);
@@ -119,12 +188,19 @@ static int cdaProcessKey(uint16_t key)
 	int i;
 	switch (key)
 	{
-		case 'p': case 'P': case KEY_CTRL_P:
+		case 'p': case 'P':
+			startpausefade();
+			break;
+		case KEY_CTRL_P:
 			plPause=!plPause;
 			if (plPause)
-				cdStop(fileno(cdpDrive));
+				cdPause(fileno(cdpDrive));
 			else
 				cdRestart(fileno(cdpDrive));
+
+			break;
+		case 't':
+			cdpViewSectors=!cdpViewSectors;
 			break;
 /*		case 0x2000:
 			cdpPlayMode=1;
@@ -133,11 +209,11 @@ static int cdaProcessKey(uint16_t key)
 			length=cdpTrackStarts[cdpTrackNum]-basesec;
 			//    strcpy(name, "DISK");
 			break;*/
-		case KEY_UP:
+		case KEY_DOWN:
 			newpos-=75;
 			setnewpos=1;
 			break;
-		case KEY_DOWN:
+		case KEY_UP:
 		    	newpos+=75;
 			setnewpos=1;
 			break;
@@ -149,7 +225,7 @@ static int cdaProcessKey(uint16_t key)
 			newpos+=75*10;
 			setnewpos=1;
 			break;
-		case KEY_HOME: //home
+		case KEY_HOME:
 			if (!cdpPlayMode)
 			{
 				newpos=0;
@@ -192,6 +268,18 @@ static int cdaProcessKey(uint16_t key)
 			newpos=0;
 			setnewpos=1;
 			break;*/
+		case KEY_F9:
+		case KEY_F11:
+			if ((speed-=finespeed)<16)
+				speed=16;
+			cdSetSpeed(speed);
+			break;
+		case KEY_F10:
+		case KEY_F12:
+			if ((speed+=finespeed)>2048)
+				speed=2048;
+			cdSetSpeed(speed);
+			break;
 		default:
 			if (smpProcessKey)
 			{
@@ -201,22 +289,40 @@ static int cdaProcessKey(uint16_t key)
 				if (ret)
 					return 1;
 			}
+			if (plrProcessKey)
+			{
+				int ret=plrProcessKey(key);
+				if (ret==2)
+					cpiResetScreen();
+				if (ret)
+					return 1;
+			}
 			return 0;
 	}
 	return 1;
 }
 
-static unsigned char cdCheck(void)
+static int cdaLooped(void)
 {
-	curpos=cdGetHeadLocation(fileno(cdpDrive), &status);
-      	if (status&STATUS_ERROR)
-	    	return 1;
-	if (!(status&STATUS_PLAY)&&!plPause&&!setnewpos)
-	{
-		setnewpos=1;
-		newpos=length;
-	}
+	struct cdStat stat;
 
+	if (pausefadedirect)
+		dopausefade();
+
+	cdSetLoop(fsLoopMods);
+	
+	cdIdle();
+
+	cdGetStatus(fileno(cdpDrive), &stat);
+
+	/*
+	if (status->error&STATUS_ERROR)
+		return 1;
+	*/
+
+	if (stat.looped)
+		return 1;
+	
 	if (setnewpos)
 	{
 		if (newpos<0)
@@ -228,36 +334,20 @@ static unsigned char cdCheck(void)
 			else
 				return 1;
 		}
-		cdStop(fileno(cdpDrive));
-		cdPlay(fileno(cdpDrive), basesec+newpos, length-newpos);
+		cdPause(fileno(cdpDrive));
+		cdRestartAt(fileno(cdpDrive), basesec+newpos /*, length-newpos*/);
 		if (plPause)
-			cdStop(fileno(cdpDrive));
+			cdPause(fileno(cdpDrive));
 		setnewpos=0;
 	} else
-		newpos=curpos-basesec;
-	return 0;
-}
-
-static int cdaLooped(void)
-{
-	static char counter=0;
-	if (!counter||setnewpos||plPause)
-	{
-		if (cdCheck())
-			return 1;
-		counter=8;
-	} else
-		counter--;
-
+		newpos=stat.position-basesec;
+	
 	return 0;
 }
 
 static void cdaCloseFile(void)
 {
-	if (!nodevice)
-		smpCloseSampler();
 	cdStop(fileno(cdpDrive));
-	/*  cdLockTray(cdpDrive, 0); */
 }
 
 static int cdaOpenFile(const char *path, struct moduleinfostruct *info, FILE *file)
@@ -266,9 +356,6 @@ static int cdaOpenFile(const char *path, struct moduleinfostruct *info, FILE *fi
 	char ext[NAME_MAX+1];
 	unsigned char tnum;
 
-      	cfCDAtLineIn=cfGetProfileBool2(cfSoundSec, "sound", "cdsamplelinein", 0, 0); /* moved from global initclose */
-
-	nodevice=!smpSample;
 	cdpDrive=file;
 
 	_splitpath(path, 0, 0, name, ext);
@@ -288,7 +375,7 @@ static int cdaOpenFile(const char *path, struct moduleinfostruct *info, FILE *fi
 
 	if (tnum!=0xFF)
 	{
-		if ((tnum<cdpFirstTrack)||(tnum>=(cdpFirstTrack+cdpTrackNum)))
+		if ((tnum<cdpFirstTrack)||(tnum>(cdpFirstTrack+cdpTrackNum)))
 			return -1;
 		cdpPlayMode=0;
 		basesec=cdpTrackStarts[tnum-cdpFirstTrack];
@@ -298,7 +385,7 @@ static int cdaOpenFile(const char *path, struct moduleinfostruct *info, FILE *fi
 			return -1;
 		cdpPlayMode=1;
 		basesec=cdpTrackStarts[0];
-		length=cdpTrackStarts[cdpTrackNum]-basesec;
+		length=cdpTrackStarts[cdpTrackNum+1]-basesec;
 	}
 
 	newpos=0;
@@ -308,27 +395,18 @@ static int cdaOpenFile(const char *path, struct moduleinfostruct *info, FILE *fi
 	plIsEnd=cdaLooped;
 	plProcessKey=cdaProcessKey;
 	plDrawGStrings=cdaDrawGStrings;
-	if (!nodevice)
-	{
-		void *buf;
-		int len;
-
-		plGetMasterSample=smpGetMasterSample;
-		plGetRealMasterVolume=smpGetRealMasterVolume;
-		smpSetSource(cfCDAtLineIn?SMP_LINEIN:SMP_CD);
-		smpSetOptions(plsmpRate, plsmpOpt);
-		if (!smpOpenSampler(&buf, &len, smpBufSize))
-			return -1;
-	}
 
 	/*  cdLockTray(cdpDrive, 1); */
 
 	strncpy(vdev, info->comment, 8);
 	vdev[7]=0;
 
-	cdPlay(fileno(cdpDrive), basesec, length);
+	if (cdPlay(fileno(cdpDrive), basesec, length))
+		return -1;
 
-  return 0;
+	normalize();
+
+	return 0;
 }
 
 struct cpifaceplayerstruct cdaPlayer = {cdaOpenFile, cdaCloseFile};
