@@ -125,6 +125,10 @@ int mpLoad669(struct gmdmodule *m, FILE *file)
 		
 		fread(&sins, sizeof(sins), 1, file);
 
+		sins.length = uint32_little (sins.length);
+		sins.loopstart = uint32_little (sins.length);
+		sins.loopend = uint32_little (sins.loopend);
+
 		ip=&m->instruments[i];
 		sp=&m->modsamples[i];
 		sip=&m->samples[i];
@@ -164,7 +168,7 @@ int mpLoad669(struct gmdmodule *m, FILE *file)
 	
 	for (t=0; t<m->patnum; t++)
 	{
-		short j;
+		int j;
 
 		uint8_t *tp;
 		uint8_t *bp;
