@@ -18,21 +18,38 @@
 #include "gmdplay.h"
 #include "stuff/err.h"
 
-static const uint8_t *ibuf;
-static uint32_t bitbuf;
+static uint8_t *ibuf;
 static uint8_t bitnum;
+static uint32_t bitlen;
 
 static inline uint16_t readbitsdmf(uint8_t n)
 {
-	uint16_t v=bitbuf&((1L<<n)-1);
-	bitbuf>>=n;
-	bitnum-=n;
-	if (bitnum<=24)
+	uint16_t retval=0;
+	int offset = 0;
+	while (n)
 	{
-		bitbuf|=(long)*ibuf++<<bitnum;
-		bitnum+=8;
+		int m=n;
+
+		if (!bitlen)
+		{
+			fprintf(stderr, "readbitsdmf: ran out of buffer\n");
+			return 0;
+		}
+
+		if (m>bitnum)
+			m=bitnum;
+		retval|=(*ibuf&((1L<<m)-1))<<offset;
+		*ibuf>>=m;
+		n-=m;
+		offset+=m;
+		if ( ! ( bitnum-=m ) )
+		{
+			bitlen--;
+			ibuf++;
+			bitnum=8;
+		}
 	}
-	return v;
+	return retval;
 }
 
 static uint16_t nodenum, lastnode;
@@ -65,14 +82,13 @@ static void readtree(void)
 		(*node)[1]=-1;
 }
 
-static void unpack0(uint8_t *ob, const void *ib, uint32_t len)
+static void unpack0(uint8_t *ob, uint8_t *ib, uint32_t len)
 {
 	uint32_t i;
 
-	ibuf=(const uint8_t*)ib;
-	bitbuf=*(uint32_t*)ibuf;
-	ibuf+=4;
-	bitnum=32;
+	ibuf=ib;
+	bitnum=8;
+	bitlen = len;
 
 	nodenum=lastnode=0;
 	readtree();
@@ -104,7 +120,7 @@ static void calctempo(uint16_t rpm, uint8_t *tempo, uint8_t *bpm)
 int mpLoadDMF(struct gmdmodule *m, FILE *file)
 {
 	struct __attribute__((packed)) {
-		uint32_t sig;
+		uint8_t sig[4];
 		uint8_t ver;
 		char tracker[8];
 		char name[30];
@@ -112,7 +128,7 @@ int mpLoadDMF(struct gmdmodule *m, FILE *file)
 		char date[3];
 	} hdr;
 	
-	uint32_t sig;
+	uint8_t sig[4];
 	uint32_t next;
 
 	uint16_t ordloop,ordnum;
@@ -145,7 +161,7 @@ int mpLoadDMF(struct gmdmodule *m, FILE *file)
 	mpReset(m);
 
   	fread(&hdr, sizeof(hdr), 1, file);
-	if (hdr.sig!=*(uint32_t *)"DDMF")
+	if (memcmp(hdr.sig, "DDMF", 4))
 		return errFormSig;
 
 	if (hdr.ver<5)
@@ -159,17 +175,19 @@ int mpLoadDMF(struct gmdmodule *m, FILE *file)
 	memcpy(m->composer, hdr.composer, 20);
 	m->composer[20]=0;
 
-	fread(&sig, sizeof(uint32_t), 1, file);
+	fread(sig, sizeof(sig), 1, file);
 	fread(&next, sizeof(uint32_t), 1, file);
+	next = uint32_little (next);
 
-	if (sig==*(uint32_t *)"INFO")
+	if (!memcmp(sig, "INFO", 4))
 	{
 		fseek(file, next, SEEK_CUR);
-		fread(&sig, sizeof(uint32_t), 1, file);
+		fread(sig, sizeof(sig), 1, file);
 		fread(&next, sizeof(uint32_t), 1, file);
+		next = uint32_little (next);
 	}
 
-	if (sig==*(uint32_t *)"CMSG")
+	if (!memcmp(sig, "CMSG", 4))
 	{
 		char waste;
 		uint16_t msglen;
@@ -200,32 +218,42 @@ int mpLoadDMF(struct gmdmodule *m, FILE *file)
 			m->message[msglen]=0;
 		}
 
-		fread(&sig, sizeof(uint32_t), 1, file);
+		fread(sig, sizeof(sig), 1, file);
 		fread(&next, sizeof(uint32_t), 1, file);
+		next = uint32_little (next);
 	}
 
-	if ((sig!=*(uint32_t *)"SEQU")||(next&1))
+	if ((memcmp(sig, "SEQU", 4))||(next&1))
 		return errFormStruc;
 
-	orders=malloc(sizeof(uint16_t)*(next-4)); /* maybe too much... */
+	orders=malloc(next-4);
 	if (!orders)
 		return errAllocMem;
-	fread(&ordloop, sizeof(int16_t), 1, file);
-	fread(&ordnum, sizeof(int16_t), 1, file);
+	fread(&ordloop, sizeof(uint16_t), 1, file);
+	ordloop = uint16_little (ordloop);
+	fread(&ordnum, sizeof(uint16_t), 1, file);
+	ordnum = uint16_little (ordnum);
 	fread(orders, next-4, 1, file);
+	{
+		int i;
+		for (i=0;i<((next-4)>>1);i++)
+			orders[i] = uint16_little (orders[i]);
+	}
 	ordnum++;
 
 	if (2*ordnum>(next-4))
-		ordnum=(next-4)/2;
+		ordnum=(next-4)/2; /* 2 = sizeof(uint16_t) */
 	if (ordloop>=ordnum)ordloop=0;
 
-	fread(&sig, sizeof(uint32_t), 1, file);
+	fread(sig, sizeof(sig), 1, file);
 	fread(&next, sizeof(uint32_t), 1, file);
+	next = uint32_little (next);
 
-	if (sig!=*(uint32_t *)"PATT")
+	if (memcmp(sig, "PATT", 4))
 		return errFormStruc;
 
 	fread(&patnum, sizeof(uint16_t), 1, file);
+	patnum = uint16_little (patnum);
 	fread(&chnnum, sizeof(uint8_t), 1, file);
 	m->channum=chnnum;
 
@@ -241,19 +269,19 @@ int mpLoadDMF(struct gmdmodule *m, FILE *file)
 	for (i=0; i<patnum; i++)
 	{
 		patadr[i]=curadr;
-		curadr+=8+*(uint32_t *)(curadr+4);
+		curadr += 8 + uint32_little (*(uint32_t *)(curadr+4));
 	}
 
 /* get the new order number */
 	nordnum=0;
 	for (i=0; i<ordnum; i++)
-		nordnum+=(*(uint16_t *)(patadr[orders[i]]+2)>256)?2:1;
+		nordnum+=(uint16_little(*(uint16_t *)((patadr[orders[i]])+2))>256)?2:1;
 
 /* relocate orders */
 	curord=nordnum;
 	for (i=ordnum-1; i>=0; i--)
 	{
-		if (*(unsigned short*)(patadr[orders[i]]+2)>256)
+		if (uint16_little(*(uint16_t *)(patadr[orders[i]]+2))>256)
 		{
 			curord-=2;
 			orders[curord]=orders[i];
@@ -301,7 +329,7 @@ int mpLoadDMF(struct gmdmodule *m, FILE *file)
 			pbeat=(*pp++)>>4;
 			if (!pbeat)
 				pbeat=8;
-			len=*(unsigned short*)pp;
+			len=uint16_little(*(uint16_t *)pp);
 			pp+=6;
 			memset(nextinfobyte, 0, 33);
 			if (len>256)
@@ -586,24 +614,20 @@ int mpLoadDMF(struct gmdmodule *m, FILE *file)
 	free(patadr);
 	free(orders);
 
-	fread(&sig, sizeof(uint32_t), 1, file);
+	fread(sig, sizeof(sig), 1, file);
 	fread(&next, sizeof(uint32_t), 1, file);
-
+	next = uint32_little (next);
 /* inst!! */
 
-	if ((sig!=*(uint32_t *)"SMPI"))
+	if (memcmp(sig, "SMPI", 4))
 		return errFormStruc;
-
 	
 	fread(&m->instnum, sizeof(uint8_t), 1, file);
 	m->modsampnum=m->sampnum=m->instnum;
-
-
 	
 	if (!mpAllocInstruments(m, m->instnum)||!mpAllocSamples(m, m->sampnum)||!mpAllocModSamples(m, m->modsampnum))
 		return errAllocMem;
 
-
 	for (i=0; i<m->instnum; i++)
 	{
 		struct gmdinstrument *ip=&m->instruments[i];
@@ -638,6 +662,12 @@ int mpLoadDMF(struct gmdmodule *m, FILE *file)
 		ip->name[namelen]=0;
 
 		fread(&smp, sizeof(smp)-((hdr.ver<8)?8:0), 1, file);
+		smp.length = uint32_little (smp.length);
+		smp.loopstart = uint32_little (smp.loopstart);
+		smp.loopend = uint32_little (smp.loopend);
+		smp.freq = uint16_little (smp.freq);
+		smp.crc32 = uint32_little (smp.crc32);
+
 		smppack[i]=!!(smp.type&0x04);
 		bit16=!!(smp.type&0x02);
 		if (smp.type&0x88)
@@ -663,10 +693,11 @@ int mpLoadDMF(struct gmdmodule *m, FILE *file)
 		sp->opt=bit16?MP_OFFSETDIV2:0;
 	}
 
-	fread(&sig, sizeof(uint32_t), 1, file);
+	fread(sig, sizeof(sig), 1, file);
 	fread(&next, sizeof(uint32_t), 1, file);
+	next = uint32_little (next);
 
-	if ((sig!=*(uint32_t *)"SMPD"))
+	if (memcmp(sig, "SMPD", 4))
 		return errFormStruc;
 
 	for (i=0; i<m->instnum; i++)
@@ -679,6 +710,7 @@ int mpLoadDMF(struct gmdmodule *m, FILE *file)
     		uint8_t *smpp;
 	
 		fread(&len, sizeof(uint32_t), 1, file);
+		len = uint32_little (len);
 
 	    	if (sp->handle==0xFFFF)
 		{
