@@ -1745,7 +1745,7 @@ void mpLockPat(int st)
 		lockpattern=-1;
 }
 
-int mpGetChanSample(int ch, short *buf, int len, int rate, int opt)
+int mpGetChanSample(int ch, short *buf, int len, uint32_t rate, int opt)
 {
 	if (tdata[ch].phys==-1)
 	{
