@@ -68,7 +68,7 @@ int loadsamplePAT( int                 file,
 	uint8_t *smpp;
 
 	read(file, &sh, sizeof(sh));
-
+	PATCHDATA_endian(&sh);
 	bit16=!!(sh.modes&1);
 	if (bit16)
 	{
@@ -84,6 +84,21 @@ int loadsamplePAT( int                 file,
 
 		lownote=(getnote(sh.low_frequency)+0x80)>>8;
 		highnote=(getnote(sh.high_frequency)+0x80)>>8;
+		if (highnote>=sizeof(ins->note))
+		{
+			fprintf(stderr, "[*.PAT loader] highnote to big\n");
+			highnote=sizeof(ins->note)-1;
+		}
+		if (lownote>=sizeof(ins->note))
+		{
+			fprintf(stderr, "[*.PAT loader] lownote to big\n");
+			lownote=sizeof(ins->note)-1;
+		}
+		if (highnote<lownote)
+		{
+			fprintf(stderr, "[*.PAT loader] highnote is smaller than lownote\n");
+			highnote=lownote;
+		}
 		for (i=lownote; i<highnote; i++)
 			if (sampused[i>>3]&(1<<(i&7)))
 				break;
@@ -166,7 +181,7 @@ int loadpatchPAT( int file,
 	*ins->name=0;
 
 	read(file, &ph, sizeof(ph));
-
+	PATCHHEADER_endian(&ph);
 	if (memcmp(ph.header, "GF1PATCH110", 12))
 	{
 		fprintf(stderr, "[*.PAT loader] Invalid header\n");
@@ -179,7 +194,7 @@ int loadpatchPAT( int file,
 	}
 
 	read(file, &ih, sizeof(ih));
-
+	INSTRUMENTDATA_endian(&ih);
 	if (ih.layers>1)
 	{
 		fprintf(stderr, "[*.PAT loader] We don't know how to handle layers (#1 = %d)\n", ih.layers);
@@ -196,7 +211,7 @@ int loadpatchPAT( int file,
 	}
 
 	read(file, &lh, sizeof(lh));
-
+	LAYERDATA_endian(&lh);
 	if (!(ins->samples=calloc(sizeof(struct msample), lh.samples)))
 		return errAllocMem;
 	if (!(*smps=calloc(sizeof(struct sampleinfo), lh.samples)))
@@ -249,7 +264,7 @@ int addpatchPAT ( int                 file,
 	s=&ins->samples[sn];
 
       	read(file, &ph, sizeof(ph));
-
+	PATCHHEADER_endian(&ph);
 	if (memcmp(ph.header, "GF1PATCH110", 12))
 	{
 		fprintf(stderr, "[*.PAT loader] Invalid version...\n");
@@ -261,7 +276,7 @@ int addpatchPAT ( int                 file,
 		return errFormStruc;
 	}
 	read(file, &ih, sizeof(ih));
-
+	INSTRUMENTDATA_endian(&ih);
 	if (ih.layers<1)
 	{
 		int q;
@@ -302,7 +317,7 @@ int addpatchPAT ( int                 file,
 	}
 
 	read(file, &lh, sizeof(lh));
-
+	LAYERDATA_endian(&lh);
 	if (lh.samples!=1)
 	{
 		fprintf(stderr, "[*.PAT loader] # Samples != 1\n");
