@@ -63,10 +63,15 @@ int it_load(struct it_module *this, FILE *file)
 	int signedsamp; /* boolean */
 	int maxchan;
 
-	uint8_t ords[256];
-	uint32_t sampoff[100];
-	uint32_t insoff[100];
-	uint32_t patoff[200];
+#define MAX_ORDERS 256
+#define MAX_SAMPLES 100
+#define MAX_INSTRUMENTS 100
+#define MAX_PATTERNS 200
+	
+	uint8_t ords[MAX_ORDERS];
+	uint32_t sampoff[MAX_SAMPLES];
+	uint32_t insoff[MAX_INSTRUMENTS];
+	uint32_t patoff[MAX_PATTERNS];
 
 	this->nchan=0;
 	this->ninst=0;
@@ -87,19 +92,59 @@ int it_load(struct it_module *this, FILE *file)
 	fseek(file, 0, SEEK_SET);
 	
 	fread(&hdr, sizeof(hdr), 1, file);
+	hdr.sig     = uint32_little (hdr.sig);
+	hdr.philite = uint16_little (hdr.philite);
+	hdr.nords   = uint16_little (hdr.nords);
+	hdr.nins    = uint16_little (hdr.nins);
+	hdr.nsmps   = uint16_little (hdr.nsmps);
+	hdr.npats   = uint16_little (hdr.npats);
+	hdr.cwtv    = uint16_little (hdr.cwtv);
+	hdr.cmwt    = uint16_little (hdr.cmwt);
+	hdr.flags   = uint16_little (hdr.flags);
+	hdr.special = uint16_little (hdr.special);
+	hdr.msglen  = uint16_little (hdr.msglen);
+	hdr.msgoff  = uint32_little (hdr.msgoff);
+	hdr._d3     = uint32_little (hdr._d3);
 	if (hdr.sig!=0x4D504D49)
 		return errFormSig;
 
 	if ((hdr.flags&4)&&(hdr.cmwt<0x200))
 		return errFormOldVer;
 
+	if (hdr.nords>MAX_ORDERS)
+	{
+		fprintf(stderr, "IT: Too many orders\n");
+		return errFormStruc;
+	}
+	if (hdr.nins>MAX_INSTRUMENTS)
+	{
+		fprintf(stderr, "IT: Too many instruments\n");
+		return errFormStruc;
+	}
+	if (hdr.nsmps>MAX_SAMPLES)
+	{
+		fprintf(stderr, "IT: Too many samples\n");
+		return errFormStruc;
+	}
+	if (hdr.npats>MAX_PATTERNS)
+	{
+		fprintf(stderr, "IT: Too many patterns\n");
+		return errFormStruc;
+	}
+	
 	signedsamp=!(hdr.cwtv<0x202);
 	this->deltapacked=(hdr.cmwt==0x215);
 
-	fread(ords, hdr.nords, 1, file);
-	fread(insoff, hdr.nins*4, 1, file);
-	fread(sampoff, hdr.nsmps*4, 1, file);
-	fread(patoff, hdr.npats*4, 1, file);
+	fread(ords, hdr.nords*sizeof(uint8_t), 1, file);
+	fread(insoff, hdr.nins*sizeof(uint32_t), 1, file);
+	fread(sampoff, hdr.nsmps*sizeof(uint32_t), 1, file);
+	fread(patoff, hdr.npats*sizeof(uint32_t), 1, file);
+	for (i=0;i<hdr.nins;i++)
+		insoff[i] = uint32_little (insoff[i]);
+	for (i=0;i<hdr.nsmps;i++)
+		sampoff[i] = uint32_little (sampoff[i]);
+	for (i=0;i<hdr.npats;i++)
+		patoff[i] = uint32_little (patoff[i]);
 
 	memcpy(this->name, hdr.name, 26);
 	for ( n=0; n<26; n++ )
@@ -127,6 +172,7 @@ int it_load(struct it_module *this, FILE *file)
 		uint16_t usage;
 		char dummy[8];
 		fread(&usage, sizeof(usage), 1, file);
+		usage = uint16_little (usage);
 		for (i=0; i<usage; i++)
 			fread(dummy, 8, 1, file);
 	}
@@ -147,7 +193,8 @@ int it_load(struct it_module *this, FILE *file)
 			for ( n=0; n<32; n++ )
 				if (!cmd[n])
 					cmd[n]=32;
-			this->midicmds[i]=malloc(sizeof(char)*(strlen(cmd)+1));
+			if (!(this->midicmds[i]=malloc(sizeof(char)*(strlen(cmd)+1))))
+				return errAllocMem;
 			strcpy(this->midicmds[i], cmd);
 			strupr(this->midicmds[i]);
 		}
@@ -252,8 +299,10 @@ int it_load(struct it_module *this, FILE *file)
 		}
 		fseek(file, patoff[k], SEEK_SET);
 
-		fread(&patlen, sizeof(patlen), 1, file);
-		fread(&patrows, sizeof(patrows), 1, file);
+		fread(&patlen, sizeof(uint16_t), 1, file);
+		patlen = uint16_little (patlen);
+		fread(&patrows, sizeof(uint16_t), 1, file);
+		patrows = uint16_little (patrows);
 		fseek(file, 4, SEEK_CUR);
 
 		if (!(patbuf=malloc(sizeof(uint8_t)*patlen)))
@@ -314,7 +363,12 @@ int it_load(struct it_module *this, FILE *file)
 			*wp++=0;
 		}
 		free(patbuf);
-		this->patterns[k]=(uint8_t *)realloc(this->patterns[k], wp-this->patterns[k]);
+		{
+			void *new;
+			if (!(new=realloc(this->patterns[k], wp-this->patterns[k])))
+				return errAllocMem;	
+			this->patterns[k]=(uint8_t *)new;
+		}
 	}
 
 	if (!maxchan)
@@ -364,6 +418,14 @@ int it_load(struct it_module *this, FILE *file)
 
 		fseek(file, sampoff[i], SEEK_SET);
 		fread(&shdr, sizeof(shdr), 1, file);
+		shdr.sig        = uint32_little (shdr.sig);
+		shdr.length     = uint32_little (shdr.length);
+		shdr.loopstart  = uint32_little (shdr.loopstart);
+		shdr.loopend    = uint32_little (shdr.loopend);
+		shdr.c5spd      = uint32_little (shdr.c5spd);
+		shdr.sloopstart = uint32_little (shdr.sloopstart);
+		shdr.sloopend   = uint32_little (shdr.sloopend);
+		shdr.off        = uint32_little (shdr.off);
 
 		sampoff[i]=shdr.off;
 		strupr(shdr.dosname);
@@ -375,7 +437,10 @@ int it_load(struct it_module *this, FILE *file)
 		sp->name[26]=0;
 
 		if (!(shdr.flags&1))
+		{
+			fprintf(stderr, "itload.c: A sample with flag 0x01 set, skipped\n");
 			continue;
+		}
 
 		sp->vol=shdr.vol;
 		sp->gvl=shdr.gvl;
@@ -475,8 +540,16 @@ int it_load(struct it_module *this, FILE *file)
 
 		if (hdr.flags&4)
 		{
+			int i, j;
 			fseek(file, insoff[k], SEEK_SET);
 			fread(&ihdr, sizeof(ihdr), 1, file);
+			ihdr.sig     = uint32_little (ihdr.sig);
+			ihdr.fadeout = uint16_little (ihdr.fadeout);
+			ihdr.tver    = uint16_little (ihdr.tver);
+			ihdr.midibnk = uint16_little (ihdr.midibnk);
+			for (i=0;i<4;i++)
+				for (j=0;j<26;j++)
+					ihdr.envs[i].pts[j].p = uint16_little (ihdr.envs[i].pts[j].p);
 		} else {
 			memset(&ihdr, 0, sizeof(ihdr));
 			ihdr.sig=ihdr.tver=ihdr.nos=ihdr._d3=0;
