@@ -208,10 +208,26 @@ static void playnote(struct itplayer *this, struct it_logchan *c, const uint8_t
 			c->retrigpos=0;
 		}
 		if (this->samples[smp-1].handle==0xFFFF)
+		{
+			fprintf(stderr, "playit/itplay.c: Assert #1\n");
+			c->pch=0; /* deallocate channel, invalid sample */
 			return;
+		}
 
 		c->pch->inst=&this->instruments[c->lastins-1];
+		if (!c->pch->inst)
+		{
+			/* should be unreachable */
+			fprintf(stderr, "playit/itplay.c: Assert #2\n");
+			c->pch=0;
+		}
 		c->pch->smp=&this->samples[smp-1];
+		if (!c->pch->smp)
+		{
+			/* should be unreachable */
+			fprintf(stderr, "playit/itplay.c: Assert #3\n");
+			c->pch=0;
+		}
 	       	c->basenote=c->pch->note=c->lastnote-cmdNNote;
 		c->realnote=c->pch->inst->keytab[c->basenote][0];
 		c->pch->noteoffset=(60-c->realnote+c->basenote)*256+c->pch->smp->normnote;
@@ -1785,7 +1801,7 @@ int getrealpos(struct itplayer *this)
 	return this->realpos;
 }
 
-int getchansample(struct itplayer *this, int ch, int16_t *buf, int len, int rate, int opt)
+int getchansample(struct itplayer *this, int ch, int16_t *buf, int len, uint32_t rate, int opt)
 {
 	int i,n;
 	int chn[64];
