@@ -159,34 +159,49 @@ typedef uint32_t  dword;
  * ----------------------------------------------------------------------
  */
 
-static dword *sourcebuffer = NULL; /* source buffer */
-static dword *srcpos       = NULL; /* actual reading position */
-static byte  srcrembits    = 0;    /* bits remaining in actual read dword */
+static uint8_t *sourcebuffer = NULL;
+static uint8_t *ibuf         = NULL; /* actual reading position */
+static uint32_t bitlen;
+static uint8_t bitnum;
 
-static dword readbits(byte b) /* reads b bits from the stream */
+static inline uint32_t readbits(uint8_t n)
 {
-	dword value;
-	if (b<=srcrembits)
+	uint32_t retval=0;
+	int offset = 0;
+	while (n)
 	{
-		value=*srcpos & ((1<<b)-1);
-		*srcpos >>= b;
-		srcrembits-=b;
-	} else {
-		dword nbits=b-srcrembits;
-		value  =  *srcpos++;
-		value |= ((*srcpos & ((1<<nbits)-1)) << srcrembits);
-		*srcpos >>= nbits;
-		srcrembits= 32-nbits;
-	}
-	return value;
-}
+		int m=n;
 
+		if (!bitlen)
+		{
+			fprintf(stderr, "readbits: ran out of buffer\n");
+			return 0;
+		}
+
+		if (m>bitnum)
+			m=bitnum;
+		retval|=(*ibuf&((1L<<m)-1))<<offset;
+		*ibuf>>=m;
+		n-=m;
+		offset+=m;
+		if ( ! ( bitnum-=m ) )
+		{
+			bitlen--;
+			ibuf++;
+			bitnum=8;
+		}
+	}
+	return retval;
+}
 static int readblock(FILE *f)  /* gets block of compressed data from file */
 {
-      	word size;
-	if (fread(&size, sizeof(size), 1, f) != 1)  /* block layout : word size, <size> bytes data */
+      	uint16_t size;
+	if (fread(&size, sizeof(uint16_t), 1, f) != 1)  /* block layout : word size, <size> bytes data */
 		return 0;
-	if (!(sourcebuffer = malloc(sizeof(dword)*((size>>2)+2))))
+	size = uint16_little (size);
+	if ( ! size )
+		return 0;
+	if (!(sourcebuffer = malloc(size)))
 		return 0;
 	if (fread(sourcebuffer, size, 1, f) != 1)
 	{
@@ -194,8 +209,9 @@ static int readblock(FILE *f)  /* gets block of compressed data from file */
 		sourcebuffer=NULL; /* Just looks better to have it present */
 		return 0;
 	}
-	srcpos=sourcebuffer;
-	srcrembits=32;
+	ibuf=sourcebuffer;
+	bitnum=8;
+	bitlen=size;
 	return 1;
 }
 
