@@ -14,7 +14,7 @@
 #include "boot/plinkman.h"
 #include "filesel/mdb.h"
 
-struct psidHeader
+struct __attribute__((packed)) psidHeader
 {
 	/* All values in big-endian order. */
 	char id[4];          /* 'PSID'                                         */
@@ -42,10 +42,7 @@ static int sidReadMemInfo(struct moduleinfostruct *m, const char *buf, size_t le
 	struct psidHeader *ph;
 
 	if (len<sizeof(struct psidHeader)+2)
-	{
-		m->modtype=mtUnRead;
 		return 0;
-	}
 
 	ph = (struct psidHeader *) buf;
 
@@ -87,11 +84,10 @@ static int sidReadMemInfo(struct moduleinfostruct *m, const char *buf, size_t le
 	if (!memcmp(buf,"SIDPLAY INFOFILE",16) && (((char *)buf)[16]==0x0a || ((char *)buf)[16]==0x0d))
 	{
 		strcpy(m->modname, "SIDPlay info file");
-		m->modtype=0xFF;
-		return 0;
+		m->modtype=mtUnRead;
+		return 1;
 	}
 	
-	m->modtype=0xFF;
 	return 0;
 }
 
