@@ -5,9 +5,9 @@ struct waveinfo
 {
 	unsigned long pos;
 	unsigned long len;
-	unsigned long rate;
-	unsigned char stereo;
-	unsigned char bit16;
+	uint32_t rate;
+	int stereo;
+	int bit16;
 };
 
 extern unsigned char wpOpenPlayer(FILE *fp, int tostereo, int tolerance);
@@ -16,11 +16,11 @@ extern void wpIdle(void);
 extern void wpSetLoop(unsigned char s);
 extern char wpLooped(void);
 extern void wpPause(unsigned char p);
-extern void wpSetAmplify(unsigned long amp);
-extern void wpSetSpeed(unsigned short sp);
+extern void wpSetAmplify(uint32_t amp);
+extern void wpSetSpeed(uint16_t sp);
 extern void wpSetVolume(unsigned char vol, signed char bal, signed char pan, unsigned char opt);
 extern void wpGetInfo(struct waveinfo *);
-extern unsigned long wpGetPos(void);
-extern void wpSetPos(signed long pos);
+extern uint32_t wpGetPos(void);
+extern void wpSetPos(int32_t pos);
 
 #endif
