@@ -54,7 +54,7 @@ static int loadmod(struct xmodule *m, FILE *file, int chan, int sig, int opt)
 	uint16_t t;
 	uint8_t *temppat;
 
-	fprintf(stderr, "loadmod enter, chan=%d sig=%d opt=%d\n", chan, sig, opt);
+/*	fprintf(stderr, "loadmod enter, chan=%d sig=%d opt=%d\n", chan, sig, opt);*/
 
 	m->envelopes=0;
 	m->samples=0;
@@ -68,11 +68,12 @@ static int loadmod(struct xmodule *m, FILE *file, int chan, int sig, int opt)
 	m->ismod=!(opt&4);
 
 	fseek(file, 1080, SEEK_SET);
-	if (fread(&l, sizeof(l), 1, file)!=1)
+	if (fread(&l, sizeof(uint32_t), 1, file)!=1)
 	{
 		fprintf(stderr, "xmlmod.c: warning: fread() failed #1\n");
 		l=0;
-	}
+	} else
+		l = uint32_little (l);
 
 	m->ninst=31;
 	m->nchan=0;
@@ -354,7 +355,7 @@ static int loadmod(struct xmodule *m, FILE *file, int chan, int sig, int opt)
 			return errAllocMem;
 /*		memset(sip->ptr, 0, sip->length);*/
 		if ((result=fread(sip->ptr, 1, sip->length, file))!=sip->length)
-			fprintf(stderr, "xmlmod.c: warning: fread() failed #8 (%d of %ld)\n", result, sip->length);
+			fprintf(stderr, "xmlmod.c: warning: fread() failed #8 (%d of %d)\n", result, (unsigned int)sip->length);
 		sp->handle=i;
 	}
 
