@@ -23,13 +23,15 @@
 #include "boot/plinkman.h"
 #include "compat.h"
 
-int memicmp(const void *s1, const void *s2, size_t n)
+int memicmp(const void *_s1, const void *_s2, size_t n)
 {
-	int i;
+	const uint8_t *s1=_s1;
+	const uint8_t *s2=_s2;
+	size_t i;
 	for (i=0;i<n;i++,s1++,s2++)
 	{
-		unsigned char u1=toupper(*(unsigned char *)s1);
-		unsigned char u2=toupper(*(unsigned char *)s2);
+		unsigned char u1=toupper(*s1);
+		unsigned char u2=toupper(*s2);
 		if (u1<u2) return -1;
 		if (u2<u1) return 1;
 	}
@@ -44,7 +46,7 @@ time_t dos_clock(void)
 }
 
 void _splitpath(const char *src, char *drive, char *path, char *file, char *ext)
-/*                             , FILE_MAX+1,  PATH_MAX+1, FILE_MAX+1, FILE_MAX+1
+/*                             , NAME_MAX+1,  PATH_MAX+1, NAME_MAX+1, NAME_MAX+1
                                  drive    :  /path/       name.egon   .tar.gz    */
 {
 	int len=0;
