@@ -98,9 +98,13 @@ static int console_init(void)
 	}
 	if (strcmp(_stdin, _stdout))
 	{
+#ifdef HAVE_X11
 		fprintf(stderr, "stdout and stdin does not come from the same device, trying just X\n");
 		if (!x11_init(st.st_rdev&0xff))
 			return 0;
+#else
+		fprintf(stderr, "stdout and stdin doesn't come from the same device\n");
+#endif
 		return -1;
 	}
 	
@@ -355,14 +359,14 @@ static int __egetch(void)
 	return 0;
 }
 
-static void __drawbar(unsigned short x, unsigned short yb, unsigned short yh, unsigned long hgt, unsigned long c)
+static void __drawbar(uint16_t x, uint16_t yb, uint16_t yh, uint32_t hgt, uint32_t c)
 {
 #ifdef CONSOLE_DEBUG
 	fprintf(stderr, "drawbar not implemented in this console driver\n");
 #endif
 }
 
-static void __idrawbar(unsigned short x, unsigned short yb, unsigned short yh, unsigned long hgt, unsigned long c)
+static void __idrawbar(uint16_t x, uint16_t yb, uint16_t yh, uint32_t hgt, uint32_t c)
 {
 #ifdef CONSOLE_DEBUG
 	fprintf(stderr, "idrawbar not implemented in this console driver\n");
