@@ -39,6 +39,7 @@
 #include "x11-common.h"
 
 static Window mRootWin;
+static unsigned short plScrRowBytes;
 
 /* "stolen" from Mplayer START */
 static void vo_hidecursor(Display * disp, Window win)
@@ -76,7 +77,7 @@ enum MODE_LINES
 	MODE_640x400,
 	MODE_640x480,
 	MODE_1024x768,
-	MODE_1280x1024,
+	MODE_1280x1024
 /*	MODE_1600x1200*/
 };
 
@@ -419,6 +420,8 @@ static void x11_common_event_loop(void)
 							case '\r':
 							case '\n': key=KEY_ALT_ENTER; break;
 							case 6: set_state(!do_fullscreen); break;
+							case 16: key=KEY_CTRL_P; break;
+							case 19: key=KEY_CTRL_S; break;
 							case 26: key=KEY_CTRL_Z; break;
 						}
 					} else if ((event.xkey.state&(ControlMask|Mod1Mask))==Mod1Mask)
@@ -433,6 +436,8 @@ static void x11_common_event_loop(void)
 							case 'G': key=KEY_ALT_G; break;
 							case 'I': key=KEY_ALT_I; break;
 							case 'L': key=KEY_ALT_L; break;
+							case 'P': key=KEY_ALT_P; break;
+							case 'S': key=KEY_ALT_S; break;
 							case 'X': key=KEY_ALT_X; break;
 							case 'Z': key=KEY_ALT_Z; break;
 						}
@@ -762,26 +767,29 @@ static void plSetTextMode(unsigned char x)
 	/* if invalid mode, set it to zero */
 	if (x>7)
 		x=0;
-	
-	if (!modelines[modes[x].modeline])
+
+	if (do_fullscreen)
 	{
-		/* scale modes upwards until we have a modeline */
-		for (i=x;i<8;i++)
-			if (modelines[modes[i].modeline])
-			{
-				x=i;
-				break;
-			}
-	}
-	if (!modelines[modes[x].modeline])
-	{
-	/* not found, scale down */
-		for (i=x;i>=0;i--)
-			if (modelines[modes[i].modeline])
-			{
-				x=i;
-				break;
-			}
+		if (!modelines[modes[x].modeline])
+		{
+			/* scale modes upwards until we have a modeline */
+			for (i=x;i<8;i++)
+				if (modelines[modes[i].modeline])
+				{
+					x=i;
+					break;
+				}
+		}
+		if (!modelines[modes[x].modeline])
+		{
+		/* not found, scale down */
+			for (i=x;i>=0;i--)
+				if (modelines[modes[i].modeline])
+				{
+					x=i;
+					break;
+				}
+		}
 	}
 	
 	plScrHeight=modes[x].chary;
@@ -838,12 +846,12 @@ static void displaystr(uint16_t y, uint16_t x, uint8_t attr, const char *str, ui
 	{
 		*p++=*str;
 		if (*str)
-			*str++;
+			str++;
 		*p++=attr;
 	}
 }
 
-static void drawbar(uint16_t x, uint16_t yb, uint16_t yh, unsigned long hgt, unsigned long c)
+static void drawbar(uint16_t x, uint16_t yb, uint16_t yh, uint32_t hgt, uint32_t c)
 {
 	char buf[60];
 	int i;
@@ -885,7 +893,7 @@ static void drawbar(uint16_t x, uint16_t yb, uint16_t yh, unsigned long hgt, uns
 	}
 }
 #ifdef PFONT_IDRAWBAR
-static void idrawbar(uint16_t x, uint16_t yb, uint16_t yh, unsigned long hgt, unsigned long c)
+static void idrawbar(uint16_t x, uint16_t yb, uint16_t yh, uint32_t hgt, uint32_t c)
 {
 	unsigned char buf[60];
 	int16_t i;
@@ -1001,6 +1009,12 @@ static void RefreshScreenText(void)
 	uint8_t *mem=vgatextram;
 	int doshape=0;
 	uint8_t save=save;
+	int precalc_linelength;
+
+	if (!window)
+		return;
+	if (!image)
+		return;
 	
 	if (curshape)
 	if (time(NULL)&1)
@@ -1018,11 +1032,14 @@ static void RefreshScreenText(void)
 			if (plUse8x16)
 			{
 				for (y=0;y<plScrHeight;y++)
+				{
+					uint8_t *scr_precalc = ((uint8_t *)image->data)+y*16*image->bytes_per_line;
 					for (x=0;x<plScrWidth;x++)
 					{
 						uint8_t f, b, a, *cp;
-						uint8_t *scr=((uint8_t *)image->data)+y*16*image->bytes_per_line+x*8;
+						uint8_t *scr=scr_precalc;
 						int i, j;
+						scr_precalc += 8;
 						cp=plFont816[*(mem++)];
 						a=*(mem++);
 						f=a&15;
@@ -1055,13 +1072,17 @@ static void RefreshScreenText(void)
 							}
 						}
 					}
+				}
 			} else {
 				for (y=0;y<plScrHeight;y++)
+				{
+					uint8_t *scr_precalc = ((uint8_t *)image->data)+y*16*image->bytes_per_line;
 					for (x=0;x<plScrWidth;x++)
 					{
 						uint8_t f, b, a, *cp;
-						uint8_t *scr=((uint8_t *)image->data)+y*8*image->bytes_per_line+x*8;
+						uint8_t *scr=scr_precalc;
 						int i, j;
+						scr_precalc += 8;
 						cp=plFont88[*(mem++)];
 						a=*(mem++);
 						f=a&15;
@@ -1082,8 +1103,8 @@ static void RefreshScreenText(void)
 							scr+=8;
 							for (i=0; i<8; i++)
 							{
-								scr-=image->bytes_per_line+8;
 								uint8_t bitmap=*cp--;
+								scr-=image->bytes_per_line+8;
 								for (j=0; j<8; j++)
 								{
 									if (bitmap&1)
@@ -1094,9 +1115,11 @@ static void RefreshScreenText(void)
 							}
 						}
 					}
+				}
 			}
 			break;
 		case 15:
+			precalc_linelength = image->bytes_per_line/sizeof(uint16_t);
 			if (plUse8x16)
 			{
 				for (y=0;y<plScrHeight;y++)
@@ -1118,7 +1141,7 @@ static void RefreshScreenText(void)
 								*scr++=(bitmap&128)?f:b;
 								bitmap<<=1;
 							}
-							scr+=image->bytes_per_line/2-8;
+							scr += precalc_linelength - 8;
 						}
 						if ((doshape==1)&&(curposy==y)&&(curposx==x))
 						{
@@ -1127,7 +1150,7 @@ static void RefreshScreenText(void)
 							for (i=0; i<16; i++)
 							{
 								uint8_t bitmap=*cp--;
-								scr-=image->bytes_per_line/2+8;
+								scr -= precalc_linelength + 8;
 								for (j=0; j<8; j++)
 								{
 									if (bitmap&1)
@@ -1158,7 +1181,7 @@ static void RefreshScreenText(void)
 								*scr++=(bitmap&128)?f:b;
 								bitmap<<=1;
 							}
-							scr+=image->bytes_per_line/2-8;
+							scr += precalc_linelength - 8;
 						}
 						if ((doshape==1)&&(curposy==y)&&(curposx==x))
 						{
@@ -1166,8 +1189,8 @@ static void RefreshScreenText(void)
 							scr+=8;
 							for (i=0; i<8; i++)
 							{
-								scr-=image->bytes_per_line/2+8;
 								uint8_t bitmap=*cp--;
+								scr -= precalc_linelength + 8;
 								for (j=0; j<8; j++)
 								{
 									if (bitmap&1)
@@ -1181,15 +1204,21 @@ static void RefreshScreenText(void)
 			}
 			break;
 		case 16:
+			precalc_linelength = image->bytes_per_line/sizeof(uint16_t);
 			if (plUse8x16)
 			{
 				for (y=0;y<plScrHeight;y++)
+				{
+					uint16_t *scr_precalc = ((uint16_t *)image->data)+y*16*image->bytes_per_line/sizeof(uint16_t);
 					for (x=0;x<plScrWidth;x++)
 					{
 						uint8_t a, *cp;
 						uint16_t f, b;
-						uint16_t *scr=((uint16_t *)image->data)+y*8*image->bytes_per_line+x*8;
+						uint16_t *scr = scr_precalc;
 						int i, j;
+
+						scr_precalc += 8;
+
 						cp=plFont816[*(mem++)];
 						a=*(mem++);
 						f=palette16[a&15];
@@ -1202,7 +1231,7 @@ static void RefreshScreenText(void)
 								*scr++=(bitmap&128)?f:b;
 								bitmap<<=1;
 							}
-							scr+=image->bytes_per_line/2-8;
+							scr += precalc_linelength - 8;
 						}
 						if ((doshape==1)&&(curposy==y)&&(curposx==x))
 						{
@@ -1211,7 +1240,7 @@ static void RefreshScreenText(void)
 							for (i=0; i<16; i++)
 							{
 								uint8_t bitmap=*cp--;
-								scr-=image->bytes_per_line/2+8;
+								scr -= precalc_linelength + 8;
 								for (j=0; j<8; j++)
 								{
 									if (bitmap&1)
@@ -1222,16 +1251,22 @@ static void RefreshScreenText(void)
 							}
 						}
 					}
+				}
 			} else {
 				for (y=0;y<plScrHeight;y++)
+				{
+					uint16_t *scr_precalc = ((uint16_t *)image->data)+y*8*image->bytes_per_line/sizeof(uint16_t);
 					for (x=0;x<plScrWidth;x++)
 					{
 						uint8_t a, *cp;
 						uint16_t f, b;
-						uint16_t *scr=((uint16_t *)image->data)+y*4*image->bytes_per_line+x*8;
+						uint16_t *scr = scr_precalc;
 						int i, j;
-						cp=plFont88[(*mem++)];
-						a=(*mem++);
+
+						scr_precalc +=8 ;
+
+						cp=plFont88[*(mem++)];
+						a=*(mem++);
 						f=palette16[a&15];
 						b=palette16[a>>4];
 						for (i=0; i<8; i++)
@@ -1242,7 +1277,7 @@ static void RefreshScreenText(void)
 								*scr++=(bitmap&128)?f:b;
 								bitmap<<=1;
 							}
-							scr+=image->bytes_per_line/2-8;
+							scr += precalc_linelength - 8;
 						}
 						if ((doshape==1)&&(curposy==y)&&(curposx==x))
 						{
@@ -1250,8 +1285,8 @@ static void RefreshScreenText(void)
 							scr+=8;
 							for (i=0; i<8; i++)
 							{
-								scr-=image->bytes_per_line/2+8;
 								uint8_t bitmap=*cp--;
+								scr -= precalc_linelength + 8;
 								for (j=0; j<8; j++)
 								{
 									if (bitmap&1)
@@ -1262,21 +1297,27 @@ static void RefreshScreenText(void)
 							}
 						}
 					}
+				}
 			}
 			break;
 		case 24:
 		case 32:
+			precalc_linelength = image->bytes_per_line/sizeof(uint32_t);
 			if (plUse8x16)
 			{
 				for (y=0;y<plScrHeight;y++)
+				{
+					uint32_t *scr_precalc = ((uint32_t *)image->data)+y*16*image->bytes_per_line/sizeof(uint32_t);
 					for (x=0;x<plScrWidth;x++)
 					{
-						uint8_t a, *cp;
 						uint32_t f, b;
-						uint32_t *scr=((uint32_t *)image->data)+y*4*image->bytes_per_line+x*8;
 						int i, j;
-						cp=plFont816[(*mem++)];
-						a=(*mem++);
+						uint8_t a, *cp;
+						uint32_t *scr = scr_precalc;
+						scr_precalc +=8 ;
+
+						cp=plFont816[*(mem++)];
+						a=*(mem++);
 						f=palette32[a&15];
 						b=palette32[a>>4];
 						for (i=0; i<16; i++)
@@ -1287,7 +1328,7 @@ static void RefreshScreenText(void)
 								*scr++=(bitmap&128)?f:b;
 								bitmap<<=1;
 							}
-							scr+=image->bytes_per_line/4-8;
+							scr += precalc_linelength - 8;
 						}
 						if ((doshape==1)&&(curposy==y)&&(curposx==x))
 						{
@@ -1296,7 +1337,7 @@ static void RefreshScreenText(void)
 							for (i=0; i<16; i++)
 							{
 								uint8_t bitmap=*cp--;
-								scr-=image->bytes_per_line/4+8;
+								scr -= precalc_linelength + 8;
 								for (j=0; j<8; j++)
 								{
 									if (bitmap&1)
@@ -1307,16 +1348,20 @@ static void RefreshScreenText(void)
 							}
 						}
 					}
+				}
 			} else {
 				for (y=0;y<plScrHeight;y++)
+				{
+					uint32_t *scr_precalc = ((uint32_t *)image->data)+y*8*image->bytes_per_line/sizeof(uint32_t);	
 					for (x=0;x<plScrWidth;x++)
 					{
-						uint8_t a, *cp;
 						uint32_t f, b;
-						uint32_t *scr=((uint32_t *)image->data)+y*2*image->bytes_per_line+x*8;
 						int i, j;
-						cp=plFont88[(*mem++)];
-						a=(*mem++);
+						uint8_t a, *cp;
+						uint32_t *scr=scr_precalc;
+						scr_precalc+=8;
+						cp=plFont88[*(mem++)];
+						a=*(mem++);
 						f=palette32[a&15];
 						b=palette32[a>>4];
 						for (i=0; i<8; i++)
@@ -1327,7 +1372,7 @@ static void RefreshScreenText(void)
 								*scr++=(bitmap&128)?f:b;
 								bitmap<<=1;
 							}
-							scr+=image->bytes_per_line/4-8;
+							scr += precalc_linelength - 8;
 						}
 						if ((doshape==1)&&(curposy==y)&&(curposx==x))
 						{
@@ -1335,8 +1380,8 @@ static void RefreshScreenText(void)
 							scr+=8;
 							for (i=0; i<8; i++)
 							{
-								scr-=image->bytes_per_line/2+8;
 								uint8_t bitmap=*cp--;
+								scr -= precalc_linelength + 8;
 								for (j=0; j<8; j++)
 								{
 									if (bitmap&1)
@@ -1347,9 +1392,9 @@ static void RefreshScreenText(void)
 							}
 						}
 					}
+				}
 			}
 			break;
-
 	}
 
 	if (doshape==2)
