@@ -10,8 +10,8 @@
 
 if [ -z "$2" ] || [ -n "$3" ]; then
 	echo -e "Usage:\n$0 /path/to/ULTRASOUND.INI /path/to/pat/files"
-fi;
-
-tmp=`echo $2|sed -e 's/\\//\\\\\\//g;s/.*/&\\\\\//;s/\\\\\/\\\\\/$/\\\\\//'`
-sed -e 's/\r//;s/[0-9]=.*/\U&/;s/PatchDir\=.*/PatchDir\='$tmp'/' "$1" > /tmp/ULTRASOUND.INI
-mv /tmp/ULTRASOUND.INI $1
+else
+	tmp=`echo $2|sed -e 's/\\//\\\\\\//g;s/.*/&\\\\\//;s/\\\\\/\\\\\/$/\\\\\//'`
+	sed -e 's/\r//;s/[0-9]=.*/\U&/;s/PatchDir\=.*/PatchDir\='$tmp'/' "$1" > /tmp/ULTRASOUND.INI
+	mv /tmp/ULTRASOUND.INI $1
+fi
